
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2001 Konrad Haenel ( alterego1@gmx.at )
 *  Copyright (C) 2000 William W. Wong ( williamw@jps.net )
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex.config;


import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.lang.reflect.*;

import phex.*;
import phex.config.*;


public class Cfg
{
	public static String		sGeneralNetwork = "<General Gnutella Network>";
	
	public GUID					mProgramClientID = new GUID();
	public String				mMyIP = "";
	public int					mListeningPort = -1;
	public int					mMaxDownload = 4;
	public int					mMaxDownloadPerIP = 2;
	public int					mDownloadMaxBandwidth = 102400;
	public int					mMaxUpload = 4;
	public int					mMaxUploadPerIP = 2;
	public int					mUploadMaxBandwidth = 102400;
	public int					mNetTTL = 7;
	public int					mNetMaxTTL = 7;
	public int					mNetMaxHostToCatch = 300;
	public int					mNetMaxSendQueue = 500;
	public int					mPingFrequency = 15000;
	public int					mPingTimeout = 60000;
	public boolean				mBehindFirewall = false;
	public int					mSearchMaxSearch = 100;
	public int					mSearchMaxConcurrent = 20;
	public String				mSearchFilter = "";
	public boolean				mSearchEnableMonitor = false;
	public int					mNetMaxConnection = 10;
	public int					mNetMaxRate = 50000;
	public int					mNetMySpeed = 28;
	public int					mNetConnectionTimeout = 30000;
	public boolean				mDownloadAutoRemoveCompleted = false;
	public String				mDownloadDir = ".";
	public int					mDownloadMaxRetry = 999;
	public int					mDownloadRetryWait = 30*1000;
	public boolean				mAutoSearchCandidate = true;
	public boolean				mAutoConnect = true;
	public int					mNetMinConn = 4;
	public boolean				mAutoCleanup = true;
	public int					mUploadMaxSearch = 100;
	public boolean				mShareEnabled = true;
	public boolean				mShareBrowseDir = true;
	public int					mPushTransferTimeout = 60*1000;
	public Vector				mNetIgnoredHosts = new Vector();
	public Vector				mNetInvalidHosts = new Vector();
	public Vector				mFilteredSearchHosts = new Vector();
	public boolean				mApplyFilterdHosts = false;
	public String				mCurrentNetwork = sGeneralNetwork;
	public Vector				mNetNetworkHistory = new Vector();
	public boolean				mAutoJoin = true;
	public Hashtable			mNetworkPasswords = new Hashtable();
	public boolean				mDisconnectApplyPolicy = true;
	public int					mDisconnectDropRatio = 50;
	public int					mDisconnectLatency = 60000;
	public boolean				mIndexFiles = false;
	public boolean				mProxyUse = false;
	public String				mProxyHost = "";
	public int					mProxyPort = 0;
	public String				mProxyUserName = "";
	public String				mProxyPassword = "";
	public Font					mFontMenu = new Font("Dialog", Font.PLAIN, 12);
	public Font					mFontLabel = new Font("Dialog", Font.PLAIN, 12);
	public Font					mFontTable = new Font("Dialog", Font.PLAIN, 12);
	public String				mFindText = "";
	public boolean				mFindMatchCase = false;
	public boolean				mFindDown = true;
	public String				mBrowserExe = "C:/Programme/Internet Explorer/IEXPLORE.EXE";
	public boolean				mUIDisplayTooltip = true;
	public String				mLFClassName = javax.swing.UIManager.getSystemLookAndFeelClassName();
	public String				mUserName = "(noname)";
	public String				mUploadDir = "";
	public String				mUploadFileExclusions = "";
	public String				mUploadFileInclusions = "*";
	public boolean				mUploadScanRecursively = true;
	public boolean				mUploadAutoRemoveCompleted = false;
	public String				mHostFile = "";
	public String				mLogFile = "";
	public String				mDownloadSaveFile = "";
	public String				mSearchLastSearch = "";
	public int					mSearchMinSpeed = 0;
	public boolean				mPhexPingResponse = true;

	public Vector				mSearchFilterTokens = new Vector();

	private String				mCfgFilename;
	private Properties			mSetting = new Properties();
	private Hashtable			mOldNames = new Hashtable();



	private Cfg()
	{
		// disable
	}


	public Cfg(String cfgFilename)
	{
		setupOldNames();
		mCfgFilename = cfgFilename;
	}


	public String getConfigFilename()
	{
		return mCfgFilename;
	}


	public void load()
	{
		try
		{
			FileInputStream		is = new FileInputStream(mCfgFilename);

			mSetting.load(is);
			is.close();
		}
		catch (Exception e)
		{
			System.out.println(e);
		}

		deserializeSimpleFields();
		deserializeComplexFields();

		breakSearchFilter();

		// ListeningPort doesn't exist.  Randomly generate one on the first time.
		if (mListeningPort == -1)
		{
			Random	random = new Random(System.currentTimeMillis());
			mListeningPort = random.nextInt();
			mListeningPort = mListeningPort < 0 ? -mListeningPort : mListeningPort;
			mListeningPort %= 8000;		// resolve: expand to 25000
			mListeningPort += 2000;
		}

	}


	public void save()
	{
		mSetting.clear();

		serializeSimpleFields();
		serializeComplexField();
		breakSearchFilter();

		try
		{
			FileOutputStream	os = new FileOutputStream(mCfgFilename);
			mSetting.save(os, "PHEX Config Values");
			os.close();
		}
		catch (IOException e)
		{
			System.out.println(e);
		}
	}


	private String get(String key)
	{
		String	value = (String)mSetting.get(key);

		if (value == null)
		{
			// Load value by old name.
			String	oldName = (String)mOldNames.get(key);
			if (oldName != null)
				value = (String)mSetting.get(oldName);
		}

		if (value != null)
			value = value.trim();
		return value;
	}


	private String get(String key, String defaultVal)
	{
		String	value = get(key);

		if (value == null)
			return defaultVal;
		return value;
	}


	private int getInt(String key, int defaultVal)
	{
		try
		{
			String	value = get(key);
			if (value == null)
				return defaultVal;
			else
				return Integer.parseInt(value);
		}
		catch (Exception e)
		{
			return defaultVal;
		}
	}


	private boolean getBool(String key, boolean defaultVal)
	{
		return get(key, defaultVal ? "true" : "false").equals("true");
	}


	private void set(String key, String value)
	{
		mSetting.put(key, value);
	}


	private void set(String key, int value)
	{
		mSetting.put(key, String.valueOf(value));
	}


	private void set(String key, boolean value)
	{
		mSetting.put(key, value ? "true" : "false");
	}


	private void breakSearchFilter()
	{
		StringTokenizer	tokens = new StringTokenizer(mSearchFilter);

		mSearchFilterTokens.removeAllElements();
		while (tokens.hasMoreTokens())
		{
			mSearchFilterTokens.addElement(tokens.nextToken());
		}
	}


	private void serializeSimpleFields()
	{
		Field[]			fields = this.getClass().getDeclaredFields();

		for (int i = 0; i < fields.length; i++)
		{
			String		name = fields[i].getName();
			int			modifiers = fields[i].getModifiers();
			Class		type = fields[i].getType();

			if (!Modifier.isPublic(modifiers) ||
				Modifier.isTransient(modifiers) ||
				Modifier.isStatic(modifiers))
			{
				continue;
			}

			try
			{
				if (type.getName().equals("int"))
				{
					set(name, fields[i].getInt(this));
				}
				else if (type.getName().equals("boolean"))
				{
					set(name, fields[i].getBoolean(this));
				}
				else if (type.getName().equals("java.lang.String"))
				{
					set(name, (String)fields[i].get(this));
				}
			}
			catch (Exception e)
			{
				System.out.println(name + " " + e);
			}
		}
	}


	private void serializeComplexField()
	{
		try
		{
			set("mProgramClientID", mProgramClientID.toHexString());

			String	ignoredHosts = "";
			for (int i = 0; i < mNetIgnoredHosts.size(); i++)
			{
				String[]	parts = (String[])mNetIgnoredHosts.elementAt(i);
				ignoredHosts += (parts[0] + "." + parts[1] + "." + parts[2] + "." + parts[3] + " ");
			}
			set("mNetIgnoredHosts", ignoredHosts);

			String	filteredSearchHosts = "";
			for (int i = 0; i < mFilteredSearchHosts.size(); i++)
			{
				String[]	parts = (String[])mFilteredSearchHosts.elementAt(i);
				filteredSearchHosts += (parts[0] + "." + parts[1] + "." + parts[2] + "." + parts[3] + " ");
			}
			set("mFilteredSearchHosts", filteredSearchHosts);

			String	invalidHosts = "";
			for (int i = 0; i < mNetInvalidHosts.size(); i++)
			{
				String[]	parts = (String[])mNetInvalidHosts.elementAt(i);
				invalidHosts += (parts[0] + "." + parts[1] + "." + parts[2] + "." + parts[3] + " ");
			}
			set("mNetInvalidHosts", invalidHosts);

			String	networkHistory = "";
			for (int i = 0; i < mNetNetworkHistory.size(); i++)
			{
				networkHistory += mNetNetworkHistory.elementAt(i) + " ";
			}
			set("mNetNetworkHistory", networkHistory);

			String			networkPasswords = "";
			Enumeration		enum = mNetworkPasswords.keys();
			while (enum.hasMoreElements())
			{
				String		network = (String)enum.nextElement();
				String		pass = (String)mNetworkPasswords.get(network);

				networkPasswords += network + ", " + pass + ", ";
			}
			set("mNetworkPasswords", networkPasswords);

			set("mFontMenu", mFontMenu.getName() + ";" + mFontMenu.getStyle() + ";" + mFontMenu.getSize());
			set("mFontLabel", mFontLabel.getName() + ";" + mFontLabel.getStyle() + ";" + mFontLabel.getSize());
			set("mFontTable", mFontTable.getName() + ";" + mFontTable.getStyle() + ";" + mFontTable.getSize());

			breakSearchFilter();
		}
		catch (Exception e)
		{
			System.out.println(e);
		}
	}


	private void deserializeSimpleFields()
	{
		Field[]			fields = this.getClass().getDeclaredFields();

		for (int i = 0; i < fields.length; i++)
		{
			String		name = fields[i].getName();
			int			modifiers = fields[i].getModifiers();
			Class		type = fields[i].getType();
			String		value;

			if (!Modifier.isPublic(modifiers) ||
				Modifier.isTransient(modifiers) ||
				Modifier.isStatic(modifiers))
			{
				continue;
			}

			try
			{
				// Load value by field name.
				value = get(name);
				if (value == null)
				{
					// Use default.
					continue;
				}

				if (type.getName().equals("int"))
				{
					fields[i].setInt(this, Integer.parseInt(value));
				}
				else if (type.getName().equals("boolean"))
				{
					fields[i].setBoolean(this, value.equals("true"));
				}
				else if (type.getName().equals("java.lang.String"))
				{
					fields[i].set(this, value);
				}
			}
			catch (Exception e)
			{
				System.out.println(name + " " + e);
			}
		}
	}


	private void deserializeComplexFields()
	{
		try
		{
			try
			{
				mProgramClientID.fromHexString(get("mProgramClientID"));
			}
			catch (Exception e)
			{
				// ignore.  take the default created value as new client ID.
			}

			String	ignoredHosts = get("mNetIgnoredHosts", "");
			{
				StringTokenizer	tokens = new StringTokenizer(ignoredHosts);
				while (tokens.hasMoreTokens())
				{
					mNetIgnoredHosts.addElement(ip2parts(tokens.nextToken()));
				}
			}

			String	filteredSearchHosts = get("mFilteredSearchHosts", "");
			if (filteredSearchHosts == null)
			{
				mFilteredSearchHosts = getDefaultFilteredHosts();
			}
			else
			{
				StringTokenizer	tokens = new StringTokenizer(filteredSearchHosts);
				while (tokens.hasMoreTokens())
				{
					mFilteredSearchHosts.addElement(ip2parts(tokens.nextToken()));
				}
			}

			String	invalidHosts = get("mNetInvalidHosts");
			if (invalidHosts == null)
			{
				mNetInvalidHosts = getDefaultInvalidHosts();
			}
			else
			{
				StringTokenizer	tokens = new StringTokenizer(invalidHosts);
				while (tokens.hasMoreTokens())
				{
					mNetInvalidHosts.addElement(ip2parts(tokens.nextToken()));
				}
			}

			String	networkHistory = get("mNetNetworkHistory", "");
			{
				StringTokenizer	tokens = new StringTokenizer(networkHistory);
				while (tokens.hasMoreTokens())
				{
					mNetNetworkHistory.addElement(tokens.nextToken());
				}
			}

			mNetworkPasswords.clear();
			String	networkPasswords = get("mNetworkPasswords", "");
			try
			{
				StringTokenizer	tokens = new StringTokenizer(networkPasswords, ",");
				while (tokens.hasMoreTokens())
				{
					String		network = tokens.nextToken().trim();
					String		pass = tokens.nextToken().trim();

					mNetworkPasswords.put(network, pass);
				}
			}
			catch (Exception e)
			{
				// ignore
			}

			String	font;
			font = get("mFontMenu");
			if (font != null)
			{
				try
				{
					StringTokenizer	tokens = new StringTokenizer(font, ";");
					mFontMenu = new Font(tokens.nextToken(),
										 Integer.parseInt(tokens.nextToken()),
										 Integer.parseInt(tokens.nextToken()));
				}
				catch (Exception e)
				{
					// ignore
				}
			}
			font = get("mFontLabel");
			if (font != null)
			{
				try
				{
					StringTokenizer	tokens = new StringTokenizer(font, ";");
					mFontLabel = new Font(tokens.nextToken(),
										 Integer.parseInt(tokens.nextToken()),
										 Integer.parseInt(tokens.nextToken()));
				}
				catch (Exception e)
				{
					// ignore
				}
			}
			font = get("mFontTable");
			if (font != null)
			{
				try
				{
					StringTokenizer	tokens = new StringTokenizer(font, ";");
					mFontTable = new Font(tokens.nextToken(),
										 Integer.parseInt(tokens.nextToken()),
										 Integer.parseInt(tokens.nextToken()));
				}
				catch (Exception e)
				{
					// ignore
				}
			}

		}
		catch (Exception e)
		{
//			System.out.println(e);
		}
	}


	private void setupOldNames()
	{
		mOldNames.put("mProgramClientID", "Program.CientID");
		mOldNames.put("mNetIgnoredHosts", "Net.IgnoredHosts");
		mOldNames.put("mNetFilteredSearchHosts", "Net.FilteredSearchHosts");
		mOldNames.put("mNetInvalidHosts", "Net.InvalidHosts");
		mOldNames.put("mNetNetworkHistory", "Net.NetworkHistory");
		
		mOldNames.put("mMyIP", "Net.MyIP");
		mOldNames.put("mListeningPort", "Net.ListeningPort");
		mOldNames.put("mMaxDownload", "Download.MaxDownload");
		mOldNames.put("mMaxDownloadPerIP", "Download.MaxDownloadPerIP");
		mOldNames.put("mDownloadMaxBandwidth", "Download.MaxBandwidth");
		mOldNames.put("mMaxUpload", "Upload.MaxUpload");
		mOldNames.put("mMaxUploadPerIP", "Upload.MaxUploadPerIP");
		mOldNames.put("mUploadMaxBandwidth", "Upload.MaxBandwidth");
		mOldNames.put("mNetTTL", "Net.TTL");
		mOldNames.put("mNetMaxTTL", "Net.MaxTTL");
		mOldNames.put("mNetMaxSendQueue", "Net.MaxSendQueue");
		mOldNames.put("mDisconnectDropRatio", "Disconnect.DropRatio");
		mOldNames.put("mDisconnectLatency", "Disconnect.Latency");
		mOldNames.put("mPingFrequency", "Net.PingFrequency");
		mOldNames.put("mPingTimeout", "Net.PingTimeout");
		mOldNames.put("mBehindFirewall", "Net.BehindFirewall");
		mOldNames.put("mSearchMaxSearch", "Search.MaxSearch");
		mOldNames.put("mSearchFilter", "Search.Filter");
		mOldNames.put("mSearchEnableMonitor", "Search.EnableMonitor");
		mOldNames.put("mNetMaxConnection", "Net.MaxConnection");
		mOldNames.put("mNetMaxRate", "Net.MaxRate");
		mOldNames.put("mNetMySpeed", "Net.MySpeed");
		mOldNames.put("mNetConnectionTimeout", "Net.ConnectionTimeout");
		mOldNames.put("mDownloadAutoRemoveCompleted", "Download.AutoRemoveCompleted");
		mOldNames.put("mUploadAutoRemoveCompleted", "Upload.AutoRemoveCompleted");
		mOldNames.put("mDownloadDir", "Download.Dir");
		mOldNames.put("mAutoSearchCandidate", "Download.AutoSearchCandidate");
		mOldNames.put("mAutoConnect", "Net.AutoConnect");
		mOldNames.put("mNetMinConn", "Net.MinConn");
		mOldNames.put("mAutoCleanup", "Net.AutoCleanup");
		mOldNames.put("mUploadMaxSearch", "Upload.MaxSearch");
		mOldNames.put("mShareEnabled", "Share.Enabled");
		mOldNames.put("mShareBrowseDir", "Share.BrowseDir");
		mOldNames.put("mPushTransferTimeout", "Download.PushTransferTimeout");
		mOldNames.put("mApplyFilterdHosts", "Net.ApplyFilterdHosts");
		mOldNames.put("mCurrentNetwork", "Net.CurrentNetwork");
		mOldNames.put("mAutoJoin", "Net.AutoJoin");
		mOldNames.put("mSaveMyPassword", "Host.SaveMyPassword");
		mOldNames.put("mIndexFiles", "Upload.IndexFiles");
		mOldNames.put("mProxyUse", "Proxy.Use");
		mOldNames.put("mProxyHost", "Proxy.Host");
		mOldNames.put("mProxyPort", "Proxy.Port");
		mOldNames.put("mProxyUserName", "Proxy.UserName");
		mOldNames.put("mProxyPassword", "Proxy.Password");
		mOldNames.put("mLFClassName", "LF.ClassName");
		mOldNames.put("mUserName", "User.Name");
		mOldNames.put("mUploadDir", "Upload.Dir");
		mOldNames.put("mUploadFileExclusions", "Upload.FileExclusions");
		mOldNames.put("mUploadFileInclusions", "Upload.FileInclusions");
		mOldNames.put("mUploadScanRecursively", "Upload.ScanRecursively");
		mOldNames.put("mUploadAutoRemoveCompleted", "Upload.AutoRemoveCompleted");
		mOldNames.put("mHostFile", "HostFile");
		mOldNames.put("mLogFile", "LogFile");
		mOldNames.put("mDownloadSaveFile", "DownloadSaveFile");
		mOldNames.put("mSearchLastSearch", "Search.LastSearch");
		mOldNames.put("mSearchMinSpeed", "Search.MinSpeed");
		mOldNames.put("mFontMenu", "Font.Menu");
		mOldNames.put("mFontLabel", "Font.Label");
		mOldNames.put("mFontTable", "Font.Table");

	}




	public static Vector getDefaultInvalidHosts()
	{
		Vector	hosts = new Vector();

		hosts.addElement(ip2parts("0.0.0.0"));
		hosts.addElement(ip2parts("255.*.*.*"));
		hosts.addElement(ip2parts("10.0.0.*"));
		hosts.addElement(ip2parts("172.16.*.*"));
		hosts.addElement(ip2parts("192.168.*.*"));

		return hosts;
	}


	public static Vector getDefaultFilteredHosts()
	{
		Vector	hosts = new Vector();

		hosts.addElement(ip2parts("0.0.0.0"));
		hosts.addElement(ip2parts("255.*.*.*"));
		hosts.addElement(ip2parts("10.0.0.*"));
		hosts.addElement(ip2parts("172.16.*.*"));
		hosts.addElement(ip2parts("192.168.*.*"));

		return hosts;
	}


	public static String[] ip2parts(String ip)
	{
		int	i = ip.indexOf(":");
		if (i != -1)
		{
			ip = ip.substring(0, i);
		}
		StringTokenizer	tokens = new StringTokenizer(ip, ".");
		String[]		parts = new String[4];

		parts[0] = tokens.nextToken();
		parts[1] = tokens.nextToken();
		parts[2] = tokens.nextToken();
		parts[3] = tokens.nextToken();

		return parts;
	}


}



