
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex.connection;


import java.io.*;
import java.net.*;
import java.util.*;

import phex.*;
import phex.config.*;
import phex.utils.*;


public class ConnectionManager
{
	public ConnectionManager()
	{
	}


	public Socket connect(String hostPort)
			throws Exception
	{
		URL		tmpUrl = new URL("http://" + hostPort);
		return connect(tmpUrl.getHost(), tmpUrl.getPort());
	}


	public Socket connect(String host, int port)
			throws Exception
	{
		return connect(host, (short)port);
	}


	public Socket connect(String host, short port)
			throws Exception
	{
		if (ServiceManager.sCfg.mProxyUse)
		{
			return connectSock5(host, (short)port);
		}

		return new Socket(host, (short)port);
	}


	public Socket connectSock5(String host, short port)
			throws Exception
	{
		Socket			sock = null;
		InputStream		is = null;
		OutputStream	os = null;

		try
		{
			sock = new Socket(ServiceManager.sCfg.mProxyHost, ServiceManager.sCfg.mProxyPort);
			is = sock.getInputStream();
			os = sock.getOutputStream();

			byte[]		buf = new byte[600];
			int			len; 

			buf[0] = (byte)0x05;
			buf[1] = (byte)0x01;
			buf[2] = (byte)0x00;
			len = 3;
			if (ServiceManager.sCfg.mProxyUserName.length() > 0)
			{
				buf[1] = (byte)0x02;
				buf[3] = (byte)0x02;
				len = 4;
			}

			os.write(buf, 0, len);
			if (is.read() != 5)
			{
				throw new IOException("Invalid response from Socks5 proxy server.");
			}

			byte		method = (byte)is.read();
			if (method == (byte)0xFF)
			{
				throw new IOException("No acceptable authentication method selected by Socks5 proxy server.");
			}
			if (method == 0x00)
			{
			}
			else if (method == 0x02)
			{
				authenticateUserPassword(is, os, buf);
			}
			else
			{
				throw new IOException("Unknown authentication method selected by Socks5 proxy server.");
			}

			len = 0;
			buf[len++] = (byte)0x05;
			buf[len++] = (byte)0x01;
			buf[len++] = (byte)0x00;
			buf[len++] = (byte)0x01;
			len = IOUtil.serializeIP(host, buf, len);
			buf[len++] = (byte)(port >> 8);
			buf[len++] = (byte)(port);
			os.write(buf, 0, len);

			int		version = is.read();
			int		status = is.read();
			is.read();
			int		atype = is.read();

			if (atype == 1)
			{
				is.read();
				is.read();
				is.read();
				is.read();
			}
			else if (atype == 3)
			{
				len = is.read();
				if (len < 0)
					len += 256;
				while (len > 0)
				{
					is.read();
					len--;
				}
			}
			else if (atype == 4)
			{
				for (int i = 0; i < 16; i++)
					is.read();
			}
			else
			{
				throw new IOException("Invalid return address type");
			}

			is.read();
			is.read();
			
			if (version == 0x05 && status == 0)
			{
				return sock;
			}
			throw new IOException("Connection failed via Socks5 proxy server.  Proxy status=" + status);
		}
		catch (Exception e)
		{
			if (is != null)
			{
				try
				{
					is.close();
				}
				catch (Exception e2)
				{
				}
			}
			if (os != null)
			{
				try
				{
					os.close();
				}
				catch (Exception e2)
				{
				}
			}
			if (sock != null)
			{
				try
				{
					sock.close();
				}
				catch (Exception e2)
				{
				}
			}

			throw new IOException("Proxy server: " + e.getMessage());
		}

	}


	private void authenticateUserPassword(InputStream is, OutputStream os, byte[] buf)
			throws Exception
	{
		int		len = 0;

		buf[len++] = (byte)0x01;
		buf[len++] = (byte)ServiceManager.sCfg.mProxyUserName.length();
		len = IOUtil.serializeString(ServiceManager.sCfg.mProxyUserName, buf, len);
		buf[len++] = (byte)ServiceManager.sCfg.mProxyPassword.length();
		len = IOUtil.serializeString(ServiceManager.sCfg.mProxyPassword, buf, len);
		os.write(buf, 0, len);

		if (is.read() == 1 && is.read() == 0)
		{
			return;
		}

		throw new IOException("Proxy server authentication failed.");
	}



}


