
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex.connection;


import java.net.*;
import java.util.*;

import phex.*;


public class ConnectionRequest extends HttpRequest
{
	public static final String		sGnutellaConnect = "GNUTELLA CONNECT";
	public static final String		sGet = "GET";
	public static final String		sGiv = "GIV";


	protected ConnectionRequest()
	{
		// disable
	}


	public ConnectionRequest(String requestLine)
			throws Exception
	{
		super(requestLine);
		String	uline = requestLine;
		String	greeting = ServiceManager.getNetworkManager().getGreeting();

		if (uline.startsWith(greeting + "/"))
		{
			// "GNUTELLA CONNECT/0.4\n\n";
			// "GNUTELLA PCONNECT name/0.4\n\n";
			setMethod(sGnutellaConnect);
			requestLine = requestLine.substring(greeting.length() + 1);
			StringTokenizer	tokens = new StringTokenizer(requestLine);

			setVersion(tokens.nextToken());

			if (tokens.hasMoreTokens())
			{
				setUri(tokens.nextToken());
			}
			
			return;
		}
		else if (uline.startsWith(sGet + " "))
		{
			// GET /get/1/foo.txt HTTP/1.1\r\n
			setMethod(sGet);
			StringTokenizer	tokens = new StringTokenizer(requestLine);
			tokens.nextToken();							// skip GET

//			setUri(URLDecoder.decode(tokens.nextToken()));
			setUri(tokens.nextToken());
			setVersion(tokens.nextToken());

			return;
		}
		else if (uline.startsWith(sGiv + " "))
		{
			// GIV <file-ref-num>:<ClientID GUID in hexdec>/<filename>\n\n
			setMethod(sGiv);
			requestLine = requestLine.substring(4);		// skip GIV
			StringTokenizer	tokens = new StringTokenizer(requestLine, ":");
			putHeader("FileRef", tokens.nextToken());
			StringTokenizer	tokens2 = new StringTokenizer(tokens.nextToken(), "/");
			putHeader("ClientID", tokens2.nextToken());
			putHeader("Filename", tokens2.nextToken());

			return;
		}

		throw new Exception("Unknown request");
	}


}

