
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package phex.dialogues;


import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.event.*;
import javax.swing.*;

import phex.*;


public class DlgConfigDownload extends JDialog implements KeyListener
{
	private BaseFrame			mFrame;
	private boolean				mCancel;
	private JTextField			mPathText;
	private JTextField			mBandwidthText;
	private JTextField			mLimitText;
	private JTextField			mLimitPerIPText;
	private JCheckBox			mRemoveCompletedCheck;
	private String				mPath;
	private int					mBandwidth;
	private int					mLimit;
	private int					mLimitPerIP;
	private boolean				mRemoveCompleted;



	public DlgConfigDownload(BaseFrame frame, String path, int bandwidth, int limit, int limitPerIP, boolean removeCompleted)
	{
		super(frame, true);
		mFrame = frame;
		mPath = path;
		mBandwidth = bandwidth;
		mLimit = limit;
		mLimitPerIP = limitPerIP;
		mRemoveCompleted = removeCompleted;
		
		setTitle("Configure Download Parameters");
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);

		// Center panel for holding the fields
		JPanel		centerPanel = new JPanel(new BorderLayout());
		centerPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(20, 20, 6, 20),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(16, 20, 20, 20)));

		JPanel		pathPanel = new JPanel(new GridLayout(3, 1, 0, 0));
		JPanel		pathInputPanel = new JPanel(new BorderLayout());
		mPathText = new JTextField(mPath);
		pathInputPanel.add(BorderLayout.CENTER, mPathText);
		JButton		pickPathButton = new JButton("Browse");
		pathInputPanel.add(BorderLayout.EAST, pickPathButton);
		pathPanel.add(new JLabel("Save New File To:"));
		pathPanel.add(pathInputPanel);
		pathPanel.add(new JLabel(" "));
        ActionListener	browseListener = new ActionListener()
		{ 
            public void actionPerformed(ActionEvent event)
			{ 
				JFileChooser	dlg = new JFileChooser(mPath);
				dlg.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				dlg.setDialogType(JFileChooser.CUSTOM_DIALOG);
				dlg.setDialogTitle("Select Directory for New File");
				if (dlg.showDialog(mFrame, "Select") == JFileChooser.APPROVE_OPTION)
				{
					mPath = dlg.getSelectedFile().getAbsolutePath();
					mPathText.setText(mPath);
				}
			}
		};
		pickPathButton.addActionListener(browseListener);

		// Bandwidth and MaxDownload panel
		JPanel		limitPanel = new JPanel(new GridLayout(1, 3, 16, 0));

		JPanel		bandwidthPanel = new JPanel(new GridLayout(2, 1, 0, 0));
		mBandwidthText = new JTextField("" + mBandwidth);
		bandwidthPanel.add(new JLabel("Max Bandwidth (Kbyte/s):"));
		bandwidthPanel.add(mBandwidthText);

		JPanel		countPanel = new JPanel(new GridLayout(2, 1, 0, 0));
		mLimitText = new JTextField("" + mLimit);
		countPanel.add(new JLabel("Simultaneous Download:"));
		countPanel.add(mLimitText);

		JPanel		perIPPanel = new JPanel(new GridLayout(2, 1, 0, 0));
		mLimitPerIPText = new JTextField("" + mLimitPerIP);
		perIPPanel.add(new JLabel("Download Per Host:"));
		perIPPanel.add(mLimitPerIPText);

		limitPanel.add(bandwidthPanel);
		limitPanel.add(countPanel);
		limitPanel.add(perIPPanel);

		// AutoRemoveCompleted panel
		JPanel		removePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 16));
		mRemoveCompletedCheck = new JCheckBox("Remove completed download automatically", mRemoveCompleted);
		removePanel.add(mRemoveCompletedCheck);

		// Option panel
		JPanel		optionPanel = new JPanel(new BorderLayout());

		optionPanel.add(BorderLayout.CENTER, limitPanel);
		optionPanel.add(BorderLayout.SOUTH, removePanel);
		
		centerPanel.add(BorderLayout.CENTER, pathPanel);
		centerPanel.add(BorderLayout.SOUTH, optionPanel);

		getContentPane().add(BorderLayout.CENTER, centerPanel);

		// Buttons panel
        JPanel			buttonPanel = new JPanel(new FlowLayout());
		JButton			ok = new JButton("OK");
        JButton			cancel = new JButton("Cancel");
        ActionListener	actionListener = new ActionListener()
		{ 
            public void actionPerformed(ActionEvent event)
			{ 
                if (!event.getActionCommand().equals("Cancel"))
				{
					if (!doOk())
						return;
				}
				else
				{
					mCancel = true;
				}
				setVisible(false);
            }
		};

        ok.addActionListener(actionListener);
		ok.setMnemonic('O');
        cancel.addActionListener(actionListener);
		cancel.setMnemonic('C');
        buttonPanel.add(ok);
		buttonPanel.add(cancel);
        getContentPane().add(BorderLayout.SOUTH, buttonPanel);

		addKeyListener(this);

		pack();

		MainFrame.centerWindow(this, new Point(0, 0));
	}


	public boolean getCancel()
	{
		return mCancel;
	}


	public String getPath()
	{
		return mPath;
	}

	
	public int getBandwidth()
	{
		return mBandwidth;
	}

	
	public int getLimit()
	{
		return mLimit;
	}


	public int getLimitPerIP()
	{
		return mLimitPerIP;
	}


	public boolean getRemoveCompleted()
	{
		return mRemoveCompleted;
	}


	private boolean doOk()
	{
		mPath = mPathText.getText();

		try
		{
			mBandwidth = Integer.parseInt(mBandwidthText.getText().trim());
			if (mBandwidth < 1)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a positive integer for bandwidth.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mBandwidthText.requestFocus();
			return false;
		}

		try
		{
			mLimit = Integer.parseInt(mLimitText.getText().trim());
			if (mLimit < 1)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a positive integer for max number of simultaneous downloads.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mLimitText.requestFocus();
			return false;
		}

		try
		{
			mLimitPerIP = Integer.parseInt(mLimitPerIPText.getText().trim());
			if (mLimitPerIP < 1)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a positive integer for max number of simultaneous downloads per host.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mLimitPerIPText.requestFocus();
			return false;
		}

		mRemoveCompleted = mRemoveCompletedCheck.isSelected();

		mCancel = false;
		
		return true;
	}


	// KeyListener implementation
	public void keyPressed(KeyEvent e)
	{
	}


	public void keyReleased(KeyEvent e)
	{
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
		{
			mCancel = true;
			setVisible(false);
		}
		else if (e.getKeyCode() == KeyEvent.VK_ENTER)
		{
			if (doOk())
			{
				mCancel = false;
				setVisible(false);
			}
		}
	}


	public void keyTyped(KeyEvent e)
	{
	}
	
}

