
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package phex.dialogues;


import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.event.*;
import javax.swing.*;

import phex.*;
import phex.config.*;


public class DlgConfigFont extends JDialog implements KeyListener
{
	private BaseFrame			mFrame;
	private boolean				mCancel;
	private JComboBox			mMenuFontCombo;
	private JComboBox			mMenuStyleCombo;
	private JTextField			mMenuSizeText;
	private JComboBox			mLabelFontCombo;
	private JComboBox			mLabelStyleCombo;
	private JTextField			mLabelSizeText;
	private JComboBox			mTableFontCombo;
	private JComboBox			mTableStyleCombo;
	private JTextField			mTableSizeText;
	private String[]			mFonts;



	public DlgConfigFont(BaseFrame frame)
	{
		super(frame, true);
		mFrame = frame;

		mFonts = mFrame.getToolkit().getFontList();

		setTitle("Configure Fonts");
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);

		// Center panel for holding the fields
		JPanel		centerPanel = new JPanel(new GridLayout(3, 1, 0, 12));
		centerPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(20, 20, 6, 20),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(16, 20, 20, 20)));

		Font		font;
		String[]	styles = {"Plain", "Bold", "Italic"};
		
		JPanel		menuPanel = new JPanel(new GridLayout(2, 3, 8, 0));
		font = ServiceManager.sCfg.mFontMenu;
		mMenuFontCombo = new JComboBox(mFonts);
		mMenuFontCombo.setSelectedItem(font.getName());
		mMenuStyleCombo = new JComboBox(styles);
		mMenuStyleCombo.setSelectedIndex(font.getStyle());
		mMenuSizeText = new JTextField("" + font.getSize());
		menuPanel.add(new JLabel("Menu Font:"));
		menuPanel.add(new JLabel("Style:"));
		menuPanel.add(new JLabel("Size:"));
		menuPanel.add(mMenuFontCombo);
		menuPanel.add(mMenuStyleCombo);
		menuPanel.add(mMenuSizeText);
		
		JPanel		labelPanel = new JPanel(new GridLayout(2, 3, 8, 0));
		font = ServiceManager.sCfg.mFontLabel;
		mLabelFontCombo = new JComboBox(mFonts);
		mLabelFontCombo.setSelectedItem(font.getName());
		mLabelStyleCombo = new JComboBox(styles);
		mLabelStyleCombo.setSelectedIndex(font.getStyle());
		mLabelSizeText = new JTextField("" + font.getSize());
		labelPanel.add(new JLabel("Label Font:"));
		labelPanel.add(new JLabel("Style:"));
		labelPanel.add(new JLabel("Size:"));
		labelPanel.add(mLabelFontCombo);
		labelPanel.add(mLabelStyleCombo);
		labelPanel.add(mLabelSizeText);
		
		JPanel		tablePanel = new JPanel(new GridLayout(2, 3, 8, 0));
		font = ServiceManager.sCfg.mFontTable;
		mTableFontCombo = new JComboBox(mFonts);
		mTableFontCombo.setSelectedItem(font.getName());
		mTableStyleCombo = new JComboBox(styles);
		mTableStyleCombo.setSelectedIndex(font.getStyle());
		mTableSizeText = new JTextField("" + font.getSize());
		tablePanel.add(new JLabel("Content Font:"));
		tablePanel.add(new JLabel("Style:"));
		tablePanel.add(new JLabel("Size:"));
		tablePanel.add(mTableFontCombo);
		tablePanel.add(mTableStyleCombo);
		tablePanel.add(mTableSizeText);

		centerPanel.add(menuPanel);
		centerPanel.add(labelPanel);
		centerPanel.add(tablePanel);

		getContentPane().add(BorderLayout.CENTER, centerPanel);

		// Buttons panel
        JPanel			buttonPanel = new JPanel(new FlowLayout());
		JButton			ok = new JButton("OK");
        JButton			cancel = new JButton("Cancel");
        ActionListener	actionListener = new ActionListener()
		{ 
            public void actionPerformed(ActionEvent event)
			{ 
                if (!event.getActionCommand().equals("Cancel"))
				{
					if (!doOk())
						return;
				}
				else
				{
					mCancel = true;
				}
				setVisible(false);
            }
		};

        ok.addActionListener(actionListener);
		ok.setMnemonic('O');
        cancel.addActionListener(actionListener);
		cancel.setMnemonic('C');
        buttonPanel.add(ok);
		buttonPanel.add(cancel);
        getContentPane().add(BorderLayout.SOUTH, buttonPanel);

		addKeyListener(this);

		pack();

		MainFrame.centerWindow(this, new Point(0, 0));
	}


	public boolean getCancel()
	{
		return mCancel;
	}


	private boolean doOk()
	{
		String	name;
		int		style;
		int		size;

		try
		{
			size = Integer.parseInt(mMenuSizeText.getText().trim());
			if (size < 1)
				throw new Exception();
			size = Integer.parseInt(mLabelSizeText.getText().trim());
			if (size < 1)
				throw new Exception();
			size = Integer.parseInt(mTableSizeText.getText().trim());
			if (size < 1)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for size.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			return false;
		}

		name = mFonts[mMenuFontCombo.getSelectedIndex()];
		style = mMenuStyleCombo.getSelectedIndex();
		size = Integer.parseInt(mMenuSizeText.getText().trim());
		ServiceManager.sCfg.mFontMenu = new Font(name, style, size);
		
		name = mFonts[mLabelFontCombo.getSelectedIndex()];
		style = mLabelStyleCombo.getSelectedIndex();
		size = Integer.parseInt(mLabelSizeText.getText().trim());
		ServiceManager.sCfg.mFontLabel = new Font(name, style, size);
		
		name = mFonts[mTableFontCombo.getSelectedIndex()];
		style = mTableStyleCombo.getSelectedIndex();
		size = Integer.parseInt(mTableSizeText.getText().trim());
		ServiceManager.sCfg.mFontTable = new Font(name, style, size);

		ServiceManager.sCfg.save();
		mCancel = false;
		return true;
	}


	// KeyListener implementation
	public void keyPressed(KeyEvent e)
	{
	}


	public void keyReleased(KeyEvent e)
	{
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
		{
			mCancel = true;
			setVisible(false);
		}
		else if (e.getKeyCode() == KeyEvent.VK_ENTER)
		{
			if (doOk())
			{
				mCancel = false;
				setVisible(false);
			}
		}
	}


	public void keyTyped(KeyEvent e)
	{
	}


}

