
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package phex.dialogues;


import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.event.*;
import javax.swing.*;

import phex.*;
import phex.config.*;
import phex.dialogues.*;


public class DlgConfigNet extends JDialog implements KeyListener
{
	private BaseFrame			mFrame;
	private boolean				mCancel;
	private JTextField			mMyIPText;
	private JTextField			mPortText;
	private JCheckBox			mBehindFirewallCheck;
	private JCheckBox			mUseProxyCheck;
	private JTextField			mSpeedText;
	private JTextField			mTTLText;
	private JTextField			mMaxTTLText;
	private JTextField			mMinConnText;
	private JTextField			mMaxConnText;
	private JTextField			mMaxRateText;
	private JTextField			mTimeoutText;
	private JTextField			mPingFrequencyText;
	private JTextField			mPingTimeoutText;
	private String				mMyIP;
	private int					mPort;
	private boolean				mBehindFirewall;
	private boolean				mUseProxy;
	private int					mSpeed;
	private int					mTTL;
	private int					mMaxTTL;
	private int					mMinConn;
	private int					mMaxConn;
	private int					mMaxRate;
	private int					mTimeout;
	private int					mPingFrequency;
	private int					mPingTimeout;



	public DlgConfigNet(BaseFrame frame)
	{
		super(frame, true);
		mFrame = frame;

		mMyIP = ServiceManager.sCfg.mMyIP;
		mPort = ServiceManager.sCfg.mListeningPort;
		mBehindFirewall = ServiceManager.sCfg.mBehindFirewall;
		mUseProxy = ServiceManager.sCfg.mProxyUse;
		mSpeed = ServiceManager.sCfg.mNetMySpeed;
		mTTL = ServiceManager.sCfg.mNetTTL;
		mMaxTTL = ServiceManager.sCfg.mNetMaxTTL;
		mMinConn = ServiceManager.sCfg.mNetMinConn;
		mMaxConn = ServiceManager.sCfg.mNetMaxConnection;
		mMaxRate = ServiceManager.sCfg.mNetMaxRate / 1024;
		mTimeout = ServiceManager.sCfg.mNetConnectionTimeout / 1000;
		mPingFrequency = ServiceManager.sCfg.mPingFrequency / 1000;
		mPingTimeout = ServiceManager.sCfg.mPingTimeout / 1000;

		setTitle("Configure Net Parameters");
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);

		// Center panel for holding the fields
		JPanel		centerPanel = new JPanel(new BorderLayout());
		centerPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(20, 20, 6, 20),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(16, 20, 20, 20)));

		mUseProxyCheck = new JCheckBox("Socks5 Proxy", mUseProxy);
		JButton	proxyButton = new JButton("Proxy Setting");
		proxyButton.addActionListener(new ProxyHandler());
		
		// Data panel
		JPanel		labelPanel = new JPanel(new GridLayout(13, 1, 10, 0));
		labelPanel.add(new JLabel("Export My Host IP As: "));
		labelPanel.add(new JLabel("Listening Port: "));
		labelPanel.add(new JLabel("Behind Firewall:"));
		labelPanel.add(mUseProxyCheck);
		labelPanel.add(new JLabel("My Host Speed: "));
		labelPanel.add(new JLabel("My TTL: "));
		labelPanel.add(new JLabel("Max TTL: "));
		labelPanel.add(new JLabel("Minimum Connections: "));
		labelPanel.add(new JLabel("Maximum Connections: "));
		labelPanel.add(new JLabel("Maximum Bandwidth (Kbyte/s): "));
		labelPanel.add(new JLabel("Connection Timeout (sec): "));
		labelPanel.add(new JLabel("Ping Frequency (sec): "));
		labelPanel.add(new JLabel("Ping Timeout (sec): "));

		JPanel		inputPanel = new JPanel(new GridLayout(13, 1, 0, 6));
		mMyIPText = new JTextField(mMyIP, 15);
		mPortText = new JTextField("" + mPort, 15);
		BaseFrame.setToolTipText(mPortText, "My listening port for other to connect to me.  Set to 0 for random port.");
		mBehindFirewallCheck = new JCheckBox("", mBehindFirewall);
		mSpeedText = new JTextField("" + mSpeed, 15);
		mTTLText = new JTextField("" + mTTL, 15);
		mMaxTTLText = new JTextField("" + mMaxTTL, 15);
		mMinConnText = new JTextField("" + mMinConn, 15);
		mMaxConnText = new JTextField("" + mMaxConn, 15);
		mMaxRateText = new JTextField("" + mMaxRate, 15);
		mTimeoutText = new JTextField("" + mTimeout, 15);
		mPingFrequencyText = new JTextField("" + mPingFrequency, 15);
		mPingTimeoutText = new JTextField("" + mPingTimeout, 15);
		inputPanel.add(mMyIPText);
		inputPanel.add(mPortText);
		inputPanel.add(mBehindFirewallCheck);
		inputPanel.add(proxyButton);
		inputPanel.add(mSpeedText);
		inputPanel.add(mTTLText);
		inputPanel.add(mMaxTTLText);
		inputPanel.add(mMinConnText);
		inputPanel.add(mMaxConnText);
		inputPanel.add(mMaxRateText);
		inputPanel.add(mTimeoutText);
		inputPanel.add(mPingFrequencyText);
		inputPanel.add(mPingTimeoutText);

		centerPanel.add(BorderLayout.WEST, labelPanel);
		centerPanel.add(BorderLayout.CENTER, inputPanel);

		getContentPane().add(BorderLayout.CENTER, centerPanel);

		// Buttons panel
        JPanel			buttonPanel = new JPanel(new FlowLayout());
		JButton			ok = new JButton("OK");
        JButton			cancel = new JButton("Cancel");
        ActionListener	actionListener = new ActionListener()
		{ 
            public void actionPerformed(ActionEvent event)
			{ 
                if (!event.getActionCommand().equals("Cancel"))
				{
					if (!doOk())
						return;
				}
				else
				{
					mCancel = true;
				}
				setVisible(false);
            }
		};

        ok.addActionListener(actionListener);
		ok.setMnemonic('O');
        cancel.addActionListener(actionListener);
		cancel.setMnemonic('C');
        buttonPanel.add(ok);
		buttonPanel.add(cancel);
        getContentPane().add(BorderLayout.SOUTH, buttonPanel);

		addKeyListener(this);

		pack();

		MainFrame.centerWindow(this, new Point(0, 0));
	}


	public boolean getCancel()
	{
		return mCancel;
	}


	String getMyIP()
	{
		return mMyIP;
	}


	int getPort()
	{
		return mPort;
	}


	boolean getBehindFirewall()
	{
		return mBehindFirewall;
	}


	boolean getUseProxy()
	{
		return mUseProxy;
	}


	int getSpeed()
	{
		return mSpeed;
	}


	int getTTL()
	{
		return mTTL;
	}


	int getMaxTTL()
	{
		return mMaxTTL;
	}


	int getMinConn()
	{
		return mMinConn;
	}


	int getMaxConn()
	{
		return mMaxConn;
	}


	int getMaxRate()
	{
		return mMaxRate;
	}


	int getTimeout()
	{
		return mTimeout;
	}


	private boolean doOk()
	{
		mMyIP = mMyIPText.getText().trim();
		
		try
		{
			mPort = Integer.parseInt(mPortText.getText().trim());
			if (mPort < 0)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for port.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mPortText.requestFocus();
			return false;
		}

		mBehindFirewall = mBehindFirewallCheck.isSelected();
		mUseProxy = mUseProxyCheck.isSelected();

		try
		{
			mSpeed = Integer.parseInt(mSpeedText.getText().trim());
			if (mSpeed < 0)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for speed.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mSpeedText.requestFocus();
			return false;
		}

		try
		{
			mTTL = Integer.parseInt(mTTLText.getText().trim());
			if (mTTL < 0)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for TTL.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mTTLText.requestFocus();
			return false;
		}

		try
		{
			mMaxTTL = Integer.parseInt(mMaxTTLText.getText().trim());
			if (mMaxTTL < 0)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for MaxTTL.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mMaxTTLText.requestFocus();
			return false;
		}

		try
		{
			mMinConn = Integer.parseInt(mMinConnText.getText().trim());
			if (mMinConn < 0)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for minimum connections.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mMinConnText.requestFocus();
			return false;
		}

		try
		{
			mMaxConn = Integer.parseInt(mMaxConnText.getText().trim());
			if (mMaxConn < 0)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for maximum connections.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mMaxConnText.requestFocus();
			return false;
		}

		try
		{
			mMaxRate = Integer.parseInt(mMaxRateText.getText().trim());
			if (mMaxRate < 0)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for max bandwidth.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mMaxRateText.requestFocus();
			return false;
		}

		try
		{
			mTimeout = Integer.parseInt(mTimeoutText.getText().trim());
			if (mTimeout < 0)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for connection timeout.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mTimeoutText.requestFocus();
			return false;
		}

		try
		{
			mPingFrequency = Integer.parseInt(mPingFrequencyText.getText().trim());
			if (mPingFrequency < 0)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for ping frequency.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mPingFrequencyText.requestFocus();
			return false;
		}

		try
		{
			mPingTimeout = Integer.parseInt(mPingTimeoutText.getText().trim());
			if (mPingTimeout < 0)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a non-negative integer for ping timeout.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mPingTimeoutText.requestFocus();
			return false;
		}

		ServiceManager.sCfg.mMyIP = mMyIP;
		ServiceManager.sCfg.mListeningPort = mPort;
		ServiceManager.sCfg.mBehindFirewall = mBehindFirewall;
		ServiceManager.sCfg.mProxyUse = mUseProxy;
		ServiceManager.sCfg.mNetMySpeed = mSpeed;
		ServiceManager.sCfg.mNetTTL = mTTL;
		ServiceManager.sCfg.mNetMaxTTL = mMaxTTL;
		ServiceManager.sCfg.mNetMinConn = mMinConn;
		ServiceManager.sCfg.mNetMaxConnection = mMaxConn;
		ServiceManager.sCfg.mNetMaxRate = mMaxRate * 1024;
		ServiceManager.sCfg.mNetConnectionTimeout = mTimeout * 1000;
		ServiceManager.sCfg.mPingFrequency = mPingFrequency * 1000;
		ServiceManager.sCfg.mPingTimeout = mPingTimeout * 1000;

		ServiceManager.sCfg.save();

		mCancel = false;
		
		return true;
	}


	// KeyListener implementation
	public void keyPressed(KeyEvent e)
	{
	}


	public void keyReleased(KeyEvent e)
	{
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
		{
			mCancel = true;
			setVisible(false);
		}
		else if (e.getKeyCode() == KeyEvent.VK_ENTER)
		{
			if (doOk())
			{
				mCancel = false;
				setVisible(false);
			}
		}
	}


	public void keyTyped(KeyEvent e)
	{
	}


	private class ProxyHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			DlgConfigProxy	dlg = new DlgConfigProxy(mFrame);

			dlg.setVisible(true);
			dlg.dispose();
		}
	}

}

