
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package phex.dialogues;


import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.event.*;
import javax.swing.*;

import phex.*;
import phex.config.*;


public class DlgConfigUI extends JDialog implements KeyListener
{
	private BaseFrame			mFrame;
	private boolean				mCancel;
	private JTextField			mPathText;
	private JCheckBox			mDisplayTooltipCheck;



	public DlgConfigUI(BaseFrame frame)
	{
		super(frame, true);
		mFrame = frame;

		setTitle("Configure UI");
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);

		JPanel		descPanel = new JPanel(new BorderLayout());
		descPanel.setBorder(BorderFactory.createEmptyBorder(10, 20, 0, 0));
		descPanel.add(BorderLayout.CENTER, new JLabel("Configure UI Setting."));

		JPanel		pathPanel = new JPanel(new GridLayout(3, 1, 0, 0));
		JPanel		pathInputPanel = new JPanel(new BorderLayout());
		mPathText = new JTextField(ServiceManager.sCfg.mBrowserExe);
		pathInputPanel.add(BorderLayout.CENTER, mPathText);
		JButton		pickPathButton = new JButton("Browse");
		pathInputPanel.add(BorderLayout.EAST, pickPathButton);
		pathPanel.add(new JLabel("Path of Web Browser Program:"));
		pathPanel.add(pathInputPanel);
		pathPanel.add(new JLabel(" "));
		ActionListener	browseListener = new ActionListener()
		{ 
			public void actionPerformed(ActionEvent event)
			{ 
				JFileChooser	dlg = new JFileChooser(mPathText.getText());
				dlg.setDialogTitle("Select Directory for New File");
				if (dlg.showDialog(mFrame, "Select") == JFileChooser.APPROVE_OPTION)
				{
					mPathText.setText(dlg.getSelectedFile().getAbsolutePath());
				}
			}
		};
		pickPathButton.addActionListener(browseListener);

		// AutoRemoveCompleted panel
		JPanel		tooltipPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 16));
		mDisplayTooltipCheck = new JCheckBox("Display popup tooltip help text (effective in next startup of PHEX).", ServiceManager.sCfg.mUIDisplayTooltip);
		tooltipPanel.add(mDisplayTooltipCheck);

		// Option panel
		JPanel		optionPanel = new JPanel(new BorderLayout());

		optionPanel.add(BorderLayout.SOUTH, tooltipPanel);
		
		// Center panel for holding the fields
		JPanel		centerPanel = new JPanel(new BorderLayout());
		centerPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(20, 20, 6, 20),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(16, 20, 20, 20)));

		centerPanel.add(BorderLayout.CENTER, pathPanel);
		centerPanel.add(BorderLayout.SOUTH, optionPanel);

		getContentPane().add(BorderLayout.CENTER, descPanel);
		getContentPane().add(BorderLayout.CENTER, centerPanel);

		// Buttons panel
        JPanel			buttonPanel = new JPanel(new FlowLayout());
		JButton			ok = new JButton("OK");
        JButton			cancel = new JButton("Cancel");
        ActionListener	actionListener = new ActionListener()
		{ 
            public void actionPerformed(ActionEvent event)
			{ 
                if (!event.getActionCommand().equals("Cancel"))
				{
					if (!doOk())
						return;
				}
				else
				{
					mCancel = true;
				}
				setVisible(false);
            }
		};

        ok.addActionListener(actionListener);
		ok.setMnemonic('O');
        cancel.addActionListener(actionListener);
		cancel.setMnemonic('C');
        buttonPanel.add(ok);
		buttonPanel.add(cancel);
        getContentPane().add(BorderLayout.SOUTH, buttonPanel);

		addKeyListener(this);

		pack();

		MainFrame.centerWindow(this, new Point(0, 0));
	}


	public boolean getCancel()
	{
		return mCancel;
	}


	private boolean doOk()
	{
		String		path = mPathText.getText();
		boolean		tooltip = mDisplayTooltipCheck.isSelected();

		ServiceManager.sCfg.mBrowserExe = path;
		ServiceManager.sCfg.mUIDisplayTooltip = tooltip;
		ServiceManager.sCfg.save();

		mCancel = false;

		return true;
	}


	// KeyListener implementation
	public void keyPressed(KeyEvent e)
	{
	}


	public void keyReleased(KeyEvent e)
	{
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
		{
			mCancel = true;
			setVisible(false);
		}
		else if (e.getKeyCode() == KeyEvent.VK_ENTER)
		{
			if (doOk())
			{
				mCancel = false;
				setVisible(false);
			}
		}
	}


	public void keyTyped(KeyEvent e)
	{
	}

}



