
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package phex.dialogues;


import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.event.*;
import javax.swing.*;

import phex.*;


public class DlgConfigUpload extends JDialog implements KeyListener
{
	private BaseFrame			mFrame;
	private boolean				mCancel;
	private JTextField			mPathText;
	private JCheckBox			mScanRecursivelyCheck;
	private JTextField			mBandwidthText;
	private JTextField			mLimitText;
	private JTextField			mLimitPerIPText;
	private JTextField			mSearchLimitText;
	private JCheckBox			mShareBrowseDirCheck;
	private JCheckBox			mRemoveCompletedCheck;
	private JTextField			mExclusionText;
	private JTextField			mInclusionText;
	private String				mPath;
	private int					mBandwidth;
	private int					mLimit;
	private int					mLimitPerIP;
	private int					mSearchLimit;
	private boolean				mRemoveCompleted;
	private String				mExclusion;
	private String				mInclusion;
	private boolean				mScanRecursively;
	private boolean				mShareBrowseDir;



	public DlgConfigUpload(BaseFrame frame,
					String path,
					int bandwidth,
					int limit,
					int limitPerIP,
					int searchLimit,
					boolean removeCompleted,
					String exclusion,
					String inclusion,
					boolean scanRecursively,
					boolean shareBrowseDir)
	{
		super(frame, true);
		mFrame = frame;
		mPath = path;
		mBandwidth = bandwidth;
		mLimit = limit;
		mLimitPerIP = limitPerIP;
		mSearchLimit = searchLimit;
		mRemoveCompleted = removeCompleted;
		mExclusion = exclusion;
		mInclusion = inclusion;
		mScanRecursively = scanRecursively;
		mShareBrowseDir = shareBrowseDir;

		setTitle("Configure Upload Parameters");
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);

		// Center panel for holding the fields
		JPanel		centerPanel = new JPanel(new BorderLayout());
		centerPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(20, 20, 6, 20),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(16, 20, 20, 20)));

		JPanel		pathPanel = new JPanel(new GridLayout(4, 1, 0, 0));
		JPanel		pathInputPanel = new JPanel(new BorderLayout());
		mPathText = new JTextField(mPath);
		JButton		pickPathButton = new JButton("Add Path");
		pathInputPanel.add(BorderLayout.CENTER, mPathText);
		pathInputPanel.add(BorderLayout.EAST, pickPathButton);
		pathPanel.add(new JLabel("Directories To Share:"));
		pathPanel.add(pathInputPanel);
		mScanRecursivelyCheck = new JCheckBox("Share all sub-directories as well.", mScanRecursively);
		pathPanel.add(mScanRecursivelyCheck);
		pathPanel.add(new JLabel(" "));
        ActionListener	browseListener = new ActionListener()
		{ 
            public void actionPerformed(ActionEvent event)
			{ 
				JFileChooser	dlg = new JFileChooser(mPath);
				dlg.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				dlg.setDialogType(JFileChooser.CUSTOM_DIALOG);
				dlg.setDialogTitle("Select Directory to Share");
				if (dlg.showDialog(mFrame, "Select") == JFileChooser.APPROVE_OPTION)
				{
					mPath = mPathText.getText();
					if (mPath.trim().length() > 0)
						mPath += ";";
					mPath += dlg.getSelectedFile().getAbsolutePath();
					mPathText.setText(mPath);
				}
			}
		};
		pickPathButton.addActionListener(browseListener);

		// Exclusion and inclusion panel
		JPanel		maskPanel = new JPanel(new BorderLayout());
		JPanel		labelPanel = new JPanel(new GridLayout(3, 1, 10, 0));
		labelPanel.add(new JLabel("Include Files: "));
		labelPanel.add(new JLabel("Exclude Files: "));
		labelPanel.add(new JLabel(" "));

		JPanel		exinclusionPanel = new JPanel(new GridLayout(3, 1, 0, 6));
		mExclusionText = new JTextField(mExclusion);
		mInclusionText = new JTextField(mInclusion);
		exinclusionPanel.add(mInclusionText);
		exinclusionPanel.add(mExclusionText);
		exinclusionPanel.add(new JLabel(" "));

		maskPanel.add(BorderLayout.WEST, labelPanel);
		maskPanel.add(BorderLayout.CENTER, exinclusionPanel);


		// Input panel
		JPanel		filePanel = new JPanel(new BorderLayout());
		filePanel.add(BorderLayout.NORTH, pathPanel);
		filePanel.add(BorderLayout.CENTER, maskPanel);

		// Bandwidth and MaxUpload panel
		JPanel		limitPanel = new JPanel(new GridLayout(1, 4, 16, 0));

		JPanel		bandwidthPanel = new JPanel(new GridLayout(2, 1, 0, 0));
		mBandwidthText = new JTextField("" + mBandwidth);
		bandwidthPanel.add(new JLabel("Max Bandwidth (KB/s):"));
		bandwidthPanel.add(mBandwidthText);

		JPanel		countPanel = new JPanel(new GridLayout(2, 1, 0, 0));
		mLimitText = new JTextField("" + mLimit);
		countPanel.add(new JLabel("Simultaneous Uploads:"));
		countPanel.add(mLimitText);

		JPanel		perIPPanel = new JPanel(new GridLayout(2, 1, 0, 0));
		mLimitPerIPText = new JTextField("" + mLimitPerIP);
		perIPPanel.add(new JLabel("Max Uploads Per Host:"));
		perIPPanel.add(mLimitPerIPText);

		JPanel		searchPanel = new JPanel(new GridLayout(2, 1, 0, 0));
		mSearchLimitText = new JTextField("" + mSearchLimit);
		searchPanel.add(new JLabel("Search Limit:"));
		searchPanel.add(mSearchLimitText);

		limitPanel.add(bandwidthPanel);
		limitPanel.add(countPanel);
		limitPanel.add(perIPPanel);
		limitPanel.add(searchPanel);

		// AutoRemoveCompleted panel
//		JPanel		removePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 16));
		JPanel		removePanel = new JPanel(new GridLayout(2, 1, 8, 0));
		mShareBrowseDirCheck = new JCheckBox("Allow browsing of directories", mShareBrowseDir);
		removePanel.add(mShareBrowseDirCheck);
		mRemoveCompletedCheck = new JCheckBox("Remove completed upload automatically", mRemoveCompleted);
		removePanel.add(mRemoveCompletedCheck);

		// Option panel
		JPanel		optionPanel = new JPanel(new BorderLayout());

		optionPanel.add(BorderLayout.CENTER, limitPanel);
		optionPanel.add(BorderLayout.SOUTH, removePanel);
		
		centerPanel.add(BorderLayout.CENTER, filePanel);
		centerPanel.add(BorderLayout.SOUTH, optionPanel);

		getContentPane().add(BorderLayout.CENTER, centerPanel);

		// Buttons panel
        JPanel			buttonPanel = new JPanel(new FlowLayout());
		JButton			ok = new JButton("OK");
        JButton			cancel = new JButton("Cancel");
        ActionListener	actionListener = new ActionListener()
		{ 
            public void actionPerformed(ActionEvent event)
			{ 
                if (!event.getActionCommand().equals("Cancel"))
				{
					if (!doOk())
						return;
				}
				else
				{
					mCancel = true;
				}
				setVisible(false);
            }
		};

        ok.addActionListener(actionListener);
		ok.setMnemonic('O');
        cancel.addActionListener(actionListener);
		cancel.setMnemonic('C');
        buttonPanel.add(ok);
		buttonPanel.add(cancel);
        getContentPane().add(BorderLayout.SOUTH, buttonPanel);

		addKeyListener(this);

		pack();

		MainFrame.centerWindow(this, new Point(0, 0));
	}


	public boolean getCancel()
	{
		return mCancel;
	}


	public String getPath()
	{
		return mPath;
	}

	
	public int getBandwidth()
	{
		return mBandwidth;
	}

	
	public int getLimit()
	{
		return mLimit;
	}


	public int getLimitPerIP()
	{
		return mLimitPerIP;
	}


	public int getSearchLimit()
	{
		return mSearchLimit;
	}


	public boolean getRemoveCompleted()
	{
		return mRemoveCompleted;
	}


	public String getExclusion()
	{
		return mExclusion;
	}


	public String getInclusion()
	{
		return mInclusion;
	}


	public boolean getScanRecursively()
	{
		return mScanRecursively;
	}


	public boolean getShareBrowseDir()
	{
		return mShareBrowseDir;
	}
	


	private boolean doOk()
	{
		mPath = mPathText.getText();

		try
		{
			mBandwidth = Integer.parseInt(mBandwidthText.getText().trim());
			if (mBandwidth < 1)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a positive integer for bandwidth.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mBandwidthText.requestFocus();
			return false;
		}

		try
		{
			mLimit = Integer.parseInt(mLimitText.getText().trim());
			if (mLimit < 1)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a positive integer for max number of simultaneous uploads.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mLimitText.requestFocus();
			return false;
		}

		try
		{
			mLimitPerIP = Integer.parseInt(mLimitPerIPText.getText().trim());
			if (mLimitPerIP < 1)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a positive integer for max number of simultaneous uploads from the same host.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mLimitPerIPText.requestFocus();
			return false;
		}

		try
		{
			mSearchLimit = Integer.parseInt(mSearchLimitText.getText().trim());
			if (mSearchLimit < 1)
				throw new Exception();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(mFrame,
										  "Please enter a positive integer for search limit.", "Invalid Input",
										  JOptionPane.ERROR_MESSAGE);
			mSearchLimitText.requestFocus();
			return false;
		}

		mScanRecursively = mScanRecursivelyCheck.isSelected();
		mInclusion = mInclusionText.getText();
		mExclusion = mExclusionText.getText();
		mRemoveCompleted = mRemoveCompletedCheck.isSelected();
		mShareBrowseDir = mShareBrowseDirCheck.isSelected();

		mCancel = false;
		
		return true;
	}


	// KeyListener implementation
	public void keyPressed(KeyEvent e)
	{
	}


	public void keyReleased(KeyEvent e)
	{
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
		{
			mCancel = true;
			setVisible(false);
		}
		else if (e.getKeyCode() == KeyEvent.VK_ENTER)
		{
			if (doOk())
			{
				mCancel = false;
				setVisible(false);
			}
		}
	}


	public void keyTyped(KeyEvent e)
	{
	}
	
}

