
/*
 *  PHEX - The pure-java Gnutella-servent.
 *  CopyRight (C) 2001 Konrad Haenel ( alterego1@gmx.at )
 *  Copyright (C) 2000 William W. Wong ( williamw@jps.net )
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex.download;


import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;

import phex.*;
import phex.config.*;
import phex.connection.*;
import phex.download.*;
import phex.host.*;
import phex.utils.*;


public class DownloadWorker implements Runnable
{
	private DownloadManager		mManager;
	private DownloadFile		mDownload = null;
	private Host				mHost = null;



	private DownloadWorker()
	{
		// disable
	}


	public DownloadWorker(DownloadManager dm)
	{
		mManager = dm;

		(new Thread(this)).start();
	}

	/**
	*	closes the connection to the romete host
	**/
	public void stopDownload()
	{
		if (mHost != null)
		{
			mHost.closeConnection();
		}
	}


	public void run()
	{
		DownloadManager			dm = ServiceManager.getDownloadManager();
		byte[]					buf = new byte[1024];
		byte[]					matchbuf = new byte[4];
		RandomAccessFile		rFile = null;
		int						len;
		int						sleep = 1;
		HostManager				hm = ServiceManager.getHostManager();

		while (true)
		{
			// Sleep and get the next file to download.
			mDownload = mManager.getNextFileToDownload(this);
			if (mDownload == null)
			{
				ServiceManager.log("DownloadWorker terminated.");
				break;
			}

			String				saveFilename = mDownload.getLocalFilename();
			String				downloadFilename = mDownload.getDownloadName();

			try
			{
				dm.incDownloadingCount();

				mDownload.appendLog("Start download.");
				
				// Check for same file name.
				if (!deleteExistingFile(saveFilename))
				{
					mDownload.setStatus(DownloadFile.sStopped);
					continue;
				}

				// <filename>.download-<seq>
				File		dfile = new File(downloadFilename);
				boolean	append = false;
				int		startPos = 0;
				long		lngStartPos = 0;

				// Check for existence of previous partial download file.
				if (dfile.exists())
				{
					append = true;
					lngStartPos = dfile.length() - 1024;
					if ( lngStartPos < 0 )
					{
						lngStartPos = 0;
					}
					Long tempLong = new Long(lngStartPos);
					startPos = tempLong.intValue();
				}

				mDownload.appendLog("position to read=" + startPos);

				// Open the file for save.
				try
				{
					mDownload.appendLog("Download name=" + downloadFilename);
					rFile = new RandomAccessFile(downloadFilename, "rw");
				}
				catch (Exception e)
				{
					String		msg = "Failed to create the file '";
					msg += saveFilename + "' to save the download content.  ";
					msg += e.getMessage();
					JOptionPane.showMessageDialog(ServiceManager.getManager().getMainFrame(),
						msg, "Failed To Create File", JOptionPane.ERROR_MESSAGE);
					throw e;
				}

				mDownload.appendLog("Connect " + mDownload.getCurrentRemoteFile().getURL());

				// Open connection to remote host.
				URL				url = new URL(mDownload.getCurrentRemoteFile().getURL());
				mHost = new Host(hm);
				mHost.setType(Host.sTypeDownload);
				mHost.setHostAddr(url.getHost() + ":" + url.getPort());
				try
				{
//					Socket		sock = new Socket(url.getHost(), url.getPort());
					Socket		sock = ServiceManager.getConnectionManager().connect(url.getHost(), url.getPort());
					mDownload.appendLog("Normal connection ok");

					mHost.setSock(sock);
					mHost.setOs(sock.getOutputStream());
					mHost.setIs(sock.getInputStream());
				}
				catch (Exception e)
				{
					mDownload.appendLog("Normal connection failed.  " + e.getMessage());
					if (ServiceManager.sCfg.mBehindFirewall)
					{
						// I am behind firewall as well.  Push request won't work.
						mDownload.appendLog("Download failed.  " + e.toString());
						throw new Exception("Download failed.  " + e.toString());
					}

					mDownload.setStatus(DownloadFile.sError, e.getMessage());
					Thread.sleep(5000);

					// Try push request.
					mDownload.appendLog("Try push request.");

					// Normal connection failed.  Try PushRequest download.
					mHost = dm.pushRequestTransfer(mHost, mDownload);

					if (mHost == null)
					{
						throw new Exception("Time out on requesting push transfer.");
					}
				}

				// User might have stopped download during the long wait during connection.
				if (mDownload.getStatus() == DownloadFile.sStopping ||
					mDownload.getStatus() == DownloadFile.sRemoved)
				{
					// Download was stopped between getNextFileToDownload() in above
					// to here.  Check for it.
					throw new Exception("Download stopped.");
				}

				// Send download initial handshake.
				String			request = mDownload.getCurrentRemoteFile().getGetRequest() +
										  "User-Agent: PHEX\r\n" +
										  mDownload.getRangeHeader((int)startPos) +
										  "\r\n";
				mDownload.appendLog("Send download handshake: " + request);
				len = IOUtil.serializeString(request, buf, 0);
				mHost.getOs().write(buf, 0, len);

				InputStream		is = mHost.getIs();
				StringBuffer	strBuf;
				String			str;

				// Look for good response.
				len = IOUtil.readToCRLF(is, buf, 1024, 0);
				strBuf = new StringBuffer();
				IOUtil.deserializeString(buf, 0, len, strBuf);
				str = strBuf.toString().toUpperCase();
				mDownload.appendLog("Remote host replies: " + str);
				if ( !str.startsWith("HTTP") )
				{
					throw new Exception("Invalid response from remote host.");
				}
				else if ( str.indexOf("503") != -1 ) 
				{
					throw new Exception("Remote host is busy.");
				}
				else if ( (str.indexOf("200 OK") == -1) && (str.indexOf("206 OK") == -1) )
				{
					throw new Exception("Negative response to download request.");
				}

				// Look for good header.
				String			server = null;
				boolean			expectedHeader = false;
				while (true)
				{
					len = IOUtil.readToCRLF(is, buf, 1024, 0);
					if (len == 0)
						break;
					strBuf = new StringBuffer();
					IOUtil.deserializeString(buf, 0, len, strBuf);
					str = strBuf.toString().toUpperCase();
					mDownload.appendLog(str);
					
					if (str.startsWith("CONTENT-LENGTH:"))
						expectedHeader = true;
					if (str.startsWith("CONTENT-RANGE:"))
						expectedHeader = true;
					if (str.startsWith("SERVER:"))
						server = str.substring(7).trim();
				}
				mDownload.setRemoteAppName(server);

				if (!expectedHeader)
				{
					throw new Exception("Invalid header from remote host.");
				}
				
				mDownload.appendLog("Downloading...");
				int	count = startPos;
				mDownload.setStartSize(count);
				mDownload.setStatus(DownloadFile.sDownloading);
				try
				{
					// Download loop.
					rFile.seek( lngStartPos );
					while (true)
					{
						len = is.read(buf);
						if (len == -1)
							break;
						rFile.write(buf, 0, len);
						count += len;
						mDownload.setTransferredSize(count);

						// Throttle control
						int	maxRate = ServiceManager.sCfg.mDownloadMaxBandwidth / dm.getDownloadingCount();
						if (mDownload.getTransferRate() > maxRate)
						{
							sleep = sleep * 2;
						}
						else if (mDownload.getTransferRate() < maxRate)
						{
							sleep = sleep / 2;
							if (sleep == 0)
								sleep = 1;
						}
						Thread.sleep(sleep);
					}
				}
				catch (Exception e2)
				{
					// Sometime the close of connection comes as an exception.
					// Check whether I have finished downloading to decide
					// whether to throw exception.
					if (count < mDownload.getCurrentRemoteFile().getFileSize())
						throw e2;
				}

				if (count < mDownload.getCurrentRemoteFile().getFileSize())
					throw new Exception("Disconnected from remote host");

				mDownload.setTransferredSize(count);
				mDownload.setStatus(DownloadFile.sCompleted);
				hm.incStatDownloadCount(1);
				mDownload.appendLog("Download completed");
			}
			catch (Exception e)
			{
				if (mDownload.getStatus() == DownloadFile.sStopping)
				{
					mDownload.setStatus(DownloadFile.sStopped);
				}
				else if (mDownload.getStatus() == DownloadFile.sRemoved)
				{
					// nothing
				}
				else
				{
					mDownload.setStatus(DownloadFile.sError, e.getMessage());
					mDownload.appendLog("Error: " + e);
				}
			}
			finally
			{
				dm.decDownloadingCount();

				if (mHost != null)
				{
					mHost.closeConnection();
					mHost = null;
				}

				if ( rFile != null )
				{
					try
					{
						rFile.close();
					}
					catch (IOException e)
					{
					}
					
					if (mDownload.getStatus() == DownloadFile.sCompleted)
					{
						(new File(downloadFilename)).renameTo(new File(saveFilename));
						mDownload.appendLog("Rename '" + downloadFilename + "' to '" + saveFilename + "'");
					}
				}
				rFile = null;

				// Remove intermediate file after it has been closed.
				File	f = new File(mDownload.getDownloadName());
				if (f.exists() &&
					(
//					 f.length() == 0 ||
					 mDownload.getStatus() == DownloadFile.sRemoved
					))
				{
					f.delete();
				}

				mDownload.setDownloadWorker(null);
				mDownload = null;
			}
		}
	}


	private boolean deleteExistingFile(String saveFilename)
			throws Exception
	{
		File	file = new File(saveFilename);

		if (file.exists())
		{
			int	option = JOptionPane.showConfirmDialog(
				ServiceManager.getManager().getMainFrame(),
				"The file '" + saveFilename + "' exists already.  Overwrite it with the download file?",
				"Confirmation",
				JOptionPane.YES_NO_CANCEL_OPTION,
				JOptionPane.INFORMATION_MESSAGE);

			if (option != JOptionPane.YES_OPTION)
			{
				// Don't proceed.
				return false;
			}
			if (file.delete() == false)
			{
				String	msg = "Failed to delete the old file '";
				msg += saveFilename + ".'";
				JOptionPane.showMessageDialog(ServiceManager.getManager().getMainFrame(),
											  msg, "Failed To Delete File", JOptionPane.ERROR_MESSAGE);
				throw new Exception("Failed To Delete File");
			}
			mDownload.appendLog("Old file deleted.");
		}

		return true;
	}


}

