
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2001 Konrad Haenel ( alterego1@gmx.at )
 *  Copyright (C) 2000 William W. Wong ( williamw@jps.net )
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package phex;


import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import phex.actions.config.*;
import phex.actions.download.*;
import phex.actions.host.*;
import phex.actions.network.*;
import phex.actions.query.*;
import phex.actions.share.*;
import phex.actions.transfer.*;
import phex.config.*;
import phex.dialogues.*;
import phex.download.*;
import phex.host.*;
import phex.interfaces.*;
import phex.msg.*;
import phex.utils.*;




public class MainFrame extends BaseFrame implements IHostChanged, IHostCaughtChanged, ISearch, IMonitorSearch, IShareChanged, IDownloadCandidate, IDownloadChanged, IFind
{
	private MsgManager			mMsgManager = ServiceManager.getMsgManager();
	private DownloadManager		mDownloadMgr = ServiceManager.getDownloadManager();
//	private IrcManager			mIrcMgr = ServiceManager.getIrcManager();
	private HostManager			mHostMgr;
	private NetworkManager		mNetworkMgr;
	private ShareManager		mShareMgr;
	private JLabel				mStatusLabel;
	private JTabbedPane			mTabs;
	private JPanel				mSearch;
	private JPanel				mMonitor;
	private JPanel				mDownload;
	private JPanel				mUpload;
	private JPanel				mChat;
	private JPanel				mIRCChat;
	private JTable				mNetTable;
	private AbstractTableModel	mNetModel;
	private TableSorter			mNetSorter;
	private JLabel				mConnectionLabel;
	private JTextField			mConnectHostText;
	private JLabel				mMyIP;
	private JCheckBox			mAutoCleanupCheck;
	private JCheckBox			mAutoConnectCheck;
	private boolean				mSearchTextTyped = true;
	private JTextField			mSearchText;
	private JTextField			mMaxSearch;
	private JTextField			mMinSpeed;
	private JButton				mSearchButton;
	private JButton				mSearchStopButton;
	private JTextField			mSearchFilterText;
	private	JCheckBox			mApplyFilterCheck;
	private JLabel				mSearchResultLabel;
	private JComboBox			mSearchResultCombo;
	private SearchComboModel	mSearchResultModel;
	private JTable				mCatcherTable;
	private AbstractTableModel	mCatcherModel;
	private JTable				mSearchTable;
	private AbstractTableModel	mSearchModel;
	private TableSorter			mSearchSorter;
	private JTable				mMonitorTable;
	private AbstractTableModel	mMonitorModel;
	private JTable				mPassiveSearchTable;
	private AbstractTableModel	mPassiveSearchModel;
	private TableSorter			mPassiveSearchSorter;
	private JTable				mDownloadTable;
	private AbstractTableModel	mDownloadModel;
	private TableSorter			mDownloadSorter;
	private JTable				mDownloadCandidate;
	private AbstractTableModel	mDownloadCandidateModel;
	private JLabel				mCandidateLabelStatus;
	private	JCheckBox			mAutoSearchCandidateCheck;
	private JTable				mUploadTable;
	private AbstractTableModel	mUploadModel;
	private TableSorter			mUploadSorter;
	private JTable				mShareFilesTable;
	private AbstractTableModel	mShareFilesModel;
	private TableSorter			mShareFilesSorter;
	private JTable				mChatTable;
	private AbstractTableModel	mChatModel;
	private TableSorter			mChatSorter;

	private javax.swing.Timer	mRefresher;
	private JTextArea			mStats;
	private JTextField			mPassiveSearch;
	private JButton				mPassiveButton;
	private JLabel				mPassiveStatus;
	private JTextField			mSendChannel;
	private JTextArea			mChatLogText;
	private JTextField			mSendText;
	private JLabel				mMyNameLabel;
	private JCheckBox			mAutoScrollChatText;
	private CellRenderer		mCellRenderer = new CellRenderer();
	private JPopupMenu			mHostPopupMenu;
	private JPopupMenu			mCatcherPopupMenu;
	private JPopupMenu			mSearchResultPopupMenu;
	private JPopupMenu			mIrcPopupMenu;
	private JPopupMenu			mIrcUserPopupMenu;
	private boolean				mInRefreshHandler = false;
	private boolean				mDownloadChanged = false;
	private boolean				mUploadChanged = false;
	private boolean				mIrcChannelChanged = false;

	private SignalPanel			mSignalPanel;




	private MainFrame()
	{
		// disable
	}

	
	public MainFrame(Applet applet, SplashWindow splash)
	{
		super("Main", applet);

		ServiceManager.getManager().setMainFrame(this);
		mHostMgr = ServiceManager.getHostManager();
		mNetworkMgr = ServiceManager.getNetworkManager();
		mShareMgr = ServiceManager.getShareManager();

		ServiceManager.sCfg.save();

		setFonts();
		setupComponents();
		setupPopupMenus();

		setupCellRender(mNetTable, mNetModel);
		setupCellRender(mDownloadTable, mDownloadModel);
		setupCellRender(mUploadTable, mUploadModel);
/*
		setupCellRender(mCatcherTable, mCatcherModel);
		setupCellRender(mSearchTable, mSearchModel);
		setupCellRender(mMonitorTable, mMonitorModel);
		setupCellRender(mPassiveSearchTable, mPassiveSearchModel);
		setupCellRender(mShareFilesTable, mShareFilesModel);
		setupCellRender(mChatTable, mChatModel);
*/

/*
		Hashtable ht = UIManager.getDefaults();
		Enumeration enum = ht.keys();
		while (enum.hasMoreElements())
		{
			Object	key = enum.nextElement();
			System.out.println(key + "=" + ht.get(key));
		}
*/

		mHostMgr.setHostChangedListener(this);
		mHostMgr.setHostCaughtChangedListener(this);
		ServiceManager.getListener().setHostChangedListener(this);

		mMsgManager.setMonitorSearchListener(this);

		mShareMgr.setShareChangedListener(this);

		mDownloadMgr.setDownloadChangedListener(this);
		mDownloadMgr.setCandidateListener(this);

		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowHandler());

		pack();
		centerAndSizeWindow(this, 7, 8);
		refreshAllActions();
//		updateIrcInfoLabel();
		resetLookAndFeel();
		setTitle();

		splash.dispose();
		setVisible(true);

		try
		{
			ServiceManager.getListener().startup();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
/*
		try
		{
			ServiceManager.getIdentd().startup();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
*/
		ServiceManager.getNetworkManager().init();
		selectCatcherTable(0);
		mDownloadMgr.init();

		mRefresher = new javax.swing.Timer(1000, new RefreshHandler());
		mRefresher.start();

		ServiceManager.getManager().startJobsRunner();
	}


	private void setupComponents()
	{
		JLabel		label;
		
		mTabs = new JTabbedPane();
		mTabs.setMinimumSize(new Dimension(50, 50));
//		mTabs.setPreferredSize(new Dimension(780, 440));

		//
		// Net Tab
		//

		JPanel	hostTablePanel = new JPanel(new BorderLayout());
		mNetModel = new NetTableModel();
		mNetSorter = new TableSorter(mNetModel);
		mNetTable = new JTable(mNetSorter);
		mNetTable.getSelectionModel().addListSelectionListener(new SelectionHandler());
		mNetTable.addMouseListener(new MouseHandler());
		mNetSorter.addMouseListenerToHeaderInTable(mNetTable);
		mConnectionLabel = new JLabel("Connections:");
		hostTablePanel.add(BorderLayout.NORTH, mConnectionLabel);
		hostTablePanel.add(BorderLayout.CENTER, new JScrollPane(mNetTable));

		JPanel	hostButtonsPanel = new JPanel(new FlowLayout());
		mConnectHostText = TextFieldFactory.newTextField("", 16);
		mConnectHostText.addKeyListener(new ConnectHostKeyHandler());
		addRefreshComponent("ActionNetworkLeave", mConnectHostText);
		label = new JLabel("My Address:");
		addRefreshComponent("ActionNetworkLeave", label);
		mMyIP = new JLabel(" ");
		addRefreshComponent("ActionNetworkLeave", mMyIP);
		JButton	connectHostButton = new JButton("Connect");
		connectHostButton.setDefaultCapable(false);
		addRefreshComponent("ActionHostConnect", connectHostButton);
		mAutoCleanupCheck = new JCheckBox("Auto Cleanup", ServiceManager.sCfg.mAutoCleanup);
		addRefreshComponent("ActionNetworkLeave", mAutoCleanupCheck);
		hostButtonsPanel.add(label);
		hostButtonsPanel.add(mMyIP);
		hostButtonsPanel.add(new JLabel("        "));
		hostButtonsPanel.add(mConnectHostText);
		hostButtonsPanel.add(connectHostButton);
		hostButtonsPanel.add(mAutoCleanupCheck);
		connectHostButton.addActionListener(new ConnectHostHandler());
		mAutoCleanupCheck.addActionListener(new AutoCleanupHandler());

		JPanel	upperPanel = new JPanel(new BorderLayout());
		upperPanel.add(BorderLayout.CENTER, hostTablePanel);
		upperPanel.add(BorderLayout.SOUTH, hostButtonsPanel);
		upperPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(8, 4, 4, 4),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(8, 8, 8, 8)));

		JPanel	lowerPanel = new JPanel(new GridLayout(1, 2, 10, 0));

		lowerPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(8, 4, 4, 4),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(8, 8, 8, 8)));

		JPanel	statPanel = new JPanel(new BorderLayout());
		mStats = new JTextArea();
		mStats.setEditable(false);
		mStats.setBorder(BorderFactory.createEtchedBorder());
				
		JButton	updateStat = new JButton("Reset");
		updateStat.setDefaultCapable(false);
		updateStat.addActionListener(new UpdateStatHandler());
		addRefreshComponent("ActionNetworkLeave", updateStat);
		JPanel	updateButtonPanel = new JPanel(new FlowLayout());
		updateButtonPanel.add(updateStat);
		statPanel.add(BorderLayout.NORTH, new JLabel("Statistics"));
		statPanel.add(BorderLayout.CENTER, mStats);
		statPanel.add(BorderLayout.SOUTH, updateButtonPanel);

		JPanel	catcherPanel = new JPanel(new BorderLayout());
		mCatcherModel = new CatcherTableModel();
		mCatcherTable = new JTable(mCatcherModel);
		mCatcherTable.getSelectionModel().addListSelectionListener(new SelectionHandler());
		mCatcherTable.addMouseListener(new MouseHandler());
		mAutoConnectCheck = new JCheckBox("Auto Connect", ServiceManager.sCfg.mAutoConnect);
		mAutoConnectCheck.addActionListener(new AutoConnectHandler());
		JButton	connectButton = new JButton("Connect");
		connectButton.setDefaultCapable(false);
		addRefreshComponent("ActionHostConnectFromCatcher", connectButton);
		JButton	resetButton = new JButton("Reset");
		resetButton.setDefaultCapable(false);
		JPanel	connectButtonPanel = new JPanel(new FlowLayout());
		connectButtonPanel.add(mAutoConnectCheck);
		connectButtonPanel.add(connectButton);
		connectButtonPanel.add(resetButton);
		catcherPanel.add(BorderLayout.NORTH, new JLabel("Host Catcher"));
		catcherPanel.add(BorderLayout.CENTER, new JScrollPane(mCatcherTable));
		catcherPanel.add(BorderLayout.SOUTH, connectButtonPanel);
		connectButton.addActionListener(new ConnectCatcherHandler());
		resetButton.addActionListener(new ResetCatcherHandler());

		lowerPanel.add(statPanel);
		lowerPanel.add(catcherPanel);

		upperPanel.setPreferredSize(new Dimension(780, 280));
		lowerPanel.setPreferredSize(new Dimension(780, 150));
		JSplitPane	splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setDividerSize(6);
		splitPane.setTopComponent(upperPanel);
		splitPane.setBottomComponent(lowerPanel);
		
		mTabs.add(" Gnutella Net ", splitPane);


		//
		// Search Tab
		//
		
		JPanel	searchInputPanel = new JPanel(new BorderLayout());
		mSearchText = TextFieldFactory.newTextField(ServiceManager.sCfg.mSearchLastSearch, 20);
		mSearchText.addKeyListener(new SearchKeyHandler());
		setToolTipText(mSearchText, "Text to search for in the Gnutella Network.");
		JPanel	searchButtonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 8, 0));
		mSearchButton = new JButton("Search");
		mSearchButton.addActionListener(new SearchHandler());
		setToolTipText(mSearchButton, "Start searching.");
		addRefreshComponent("ActionQuerySearch", mSearchButton);
		searchButtonPanel.add(mSearchButton);
		searchInputPanel.add(BorderLayout.CENTER, mSearchText);
//		searchInputPanel.add(BorderLayout.EAST, mSearchButton);
		searchInputPanel.add(BorderLayout.EAST, searchButtonPanel);

		JPanel	maxSearchPanel = new JPanel(new BorderLayout());
		String	lastMaxSearch = "" + ServiceManager.sCfg.mSearchMaxSearch;
		mMaxSearch = TextFieldFactory.newTextField(lastMaxSearch, 8);
		mMaxSearch.addKeyListener(new SearchKeyHandler());
		setToolTipText(mMaxSearch, "The maximum number of results to display.  The rest are discarded.");
		maxSearchPanel.add(BorderLayout.WEST, new JLabel("Maximum Results: "));
		maxSearchPanel.add(BorderLayout.CENTER, mMaxSearch);
				
		JPanel	minSpeedPanel = new JPanel(new BorderLayout());
		String	lastMinSpeed = String.valueOf(ServiceManager.sCfg.mSearchMinSpeed);
		mMinSpeed = TextFieldFactory.newTextField(lastMinSpeed, 8);
		mMinSpeed.addKeyListener(new SearchKeyHandler());
		setToolTipText(mMinSpeed, "The hosts have to meet the minimum speed before returning the matching results.");
		minSpeedPanel.add(BorderLayout.WEST, new JLabel("Minimum Speed (Kb/s): "));
		minSpeedPanel.add(BorderLayout.CENTER, mMinSpeed);

		JPanel	searchFilterPanel = new JPanel(new BorderLayout());
		mSearchFilterText = TextFieldFactory.newTextField(ServiceManager.sCfg.mSearchFilter, 16);
		mSearchFilterText.addKeyListener(new SearchKeyHandler());
		setToolTipText(mSearchFilterText, "Results coming back containing one of these words will be filtered out.");
		searchFilterPanel.add(BorderLayout.WEST, new JLabel("Result Filter: "));
		searchFilterPanel.add(BorderLayout.CENTER, mSearchFilterText);
				
		JPanel	applyFilterPanel = new JPanel(new BorderLayout());
		mApplyFilterCheck = new JCheckBox("Filtered Hosts", ServiceManager.sCfg.mApplyFilterdHosts);
		setToolTipText(mApplyFilterCheck, "Filter the results whose IP addresses are in the Filtered Host List.");
		applyFilterPanel.add(BorderLayout.CENTER, mApplyFilterCheck);
		mApplyFilterCheck.addActionListener(new ApplyFilterHandler());
				
		JPanel	searchInputPanel2 = new JPanel(new GridLayout(1, 3, 16, 0));
		searchInputPanel2.add(maxSearchPanel);
		searchInputPanel2.add(minSpeedPanel);
		searchInputPanel2.add(searchFilterPanel);
		searchInputPanel2.add(applyFilterPanel);
		searchInputPanel2.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));

		JPanel	searchTopPanel = new JPanel(new BorderLayout());
		searchTopPanel.add(BorderLayout.NORTH, searchInputPanel);
		searchTopPanel.add(BorderLayout.CENTER, searchInputPanel2);
		searchTopPanel.add(BorderLayout.SOUTH, new JLabel(" "));
		searchTopPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(8, 8, 8, 8),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(8, 8, 8, 8)));

		JPanel	resultSelectPanel = new JPanel(new BorderLayout());
		mSearchResultModel = new SearchComboModel();
		mSearchResultCombo = new JComboBox(mSearchResultModel);
		mSearchResultCombo.setEditable(false);
		mSearchResultCombo.addActionListener(new SearchComboActionHandler());
		setToolTipText(mSearchResultCombo, "The list of searches.  Select one to display to result of it.");
		mSearchStopButton = new JButton("Stop");
		setToolTipText(mSearchStopButton, "Stop the selected search.");
		mSearchStopButton.setEnabled(false);
		mSearchStopButton.addActionListener(new stopSearchHandler());
		resultSelectPanel.add(BorderLayout.CENTER, mSearchResultCombo);
		resultSelectPanel.add(BorderLayout.EAST, mSearchStopButton);

		JPanel	resultTablePanel = new JPanel(new BorderLayout());
		mSearchModel = new SearchTableModel();
		mSearchSorter = new TableSorter(mSearchModel);
		mSearchTable = new JTable(mSearchSorter);
		mSearchTable.getSelectionModel().addListSelectionListener(new SelectionHandler());
		setToolTipText(mSearchTable, "Search results.  Perform sorting by clicking on the column.  press SHIFT for reverse.");
		mSearchTable.addMouseListener(new MouseHandler());				
		mSearchSorter.addMouseListenerToHeaderInTable(mSearchTable);
		mSearchResultLabel = new JLabel("Search Result:");

		JPanel	searchButtonsPanel = new JPanel(new FlowLayout());
		JButton	downloadButton = new JButton("Download");
		setToolTipText(downloadButton, "Start download on the selected files.");
		downloadButton.addActionListener(new DownloadHandler());
		addRefreshComponent("ActionTransferDownload", downloadButton);
		JButton	asCandidateButton = new JButton("Add As Candidate");
		setToolTipText(asCandidateButton, "Add the selected file as download candidate to existing download.");
		asCandidateButton.addActionListener(new AddAsCandidateHandler());
		addRefreshComponent("ActionTransferAddAsCandidate", asCandidateButton);
		searchButtonsPanel.add(downloadButton);
		searchButtonsPanel.add(asCandidateButton);

		resultTablePanel.add(BorderLayout.NORTH, mSearchResultLabel);
		resultTablePanel.add(BorderLayout.CENTER, new JScrollPane(mSearchTable));
		resultTablePanel.add(BorderLayout.SOUTH, searchButtonsPanel);

		JPanel	resultPanel = new JPanel(new BorderLayout());
		resultPanel.add(BorderLayout.NORTH, resultSelectPanel);
		resultPanel.add(BorderLayout.CENTER, resultTablePanel);
		resultPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(8, 8, 8, 8),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(8, 8, 8, 8)));

		JPanel	searchMainPanel = new JPanel(new BorderLayout());
		searchMainPanel.add(BorderLayout.NORTH, searchTopPanel);
		searchMainPanel.add(BorderLayout.CENTER, resultPanel);

		mSearch = new JPanel(new BorderLayout());
		mSearch.add(BorderLayout.CENTER, searchMainPanel);
		mSearch.setBorder(BorderFactory.createLoweredBevelBorder());

		mTabs.add(" Search ", mSearch);

		//
		//  Monitor Tab
		//

		JPanel	monitorTablePanel = new JPanel(new BorderLayout());
		mMonitorModel = new MonitorTableModel();
		mMonitorTable = new JTable(mMonitorModel);
		mMonitorTable.getSelectionModel().addListSelectionListener(new SelectionHandler());
		monitorTablePanel.add(BorderLayout.NORTH, new JLabel("Search Monitor:"));
		monitorTablePanel.add(BorderLayout.CENTER, new JScrollPane(mMonitorTable));
		monitorTablePanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(8, 8, 8, 8),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(8, 8, 8, 8)));

		JPanel	passiveSearchPanel = new JPanel(new BorderLayout());
		mPassiveSearchModel = new PassiveSearchTableModel();
		mPassiveSearchSorter = new TableSorter(mPassiveSearchModel);
		mPassiveSearchTable = new JTable(mPassiveSearchSorter);
		mPassiveSearchTable.getSelectionModel().addListSelectionListener(new SelectionHandler());
		mPassiveSearchTable.addMouseListener(new MouseHandler());
		mPassiveSearchSorter.addMouseListenerToHeaderInTable(mPassiveSearchTable);
		
		JPanel	passiveButtonPanel = new JPanel(new BorderLayout());
		passiveButtonPanel.setBorder(BorderFactory.createEmptyBorder(8, 0, 0, 0));
		mPassiveSearch = TextFieldFactory.newTextField("", 20);
		setToolTipText(mPassiveSearch, "Specify keywords to match, separated by space.");
		mPassiveButton = new JButton("Start Passive Search");
		setToolTipText(mPassiveButton, "Passive search monitors the search results routed through here.  This might take a while.");
		mPassiveStatus = new JLabel("Passive Search Result");

		mPassiveButton.addActionListener(new PassiveSearchHandler());
		passiveButtonPanel.add(BorderLayout.WEST, new JLabel("Monitor Filter:"));
		passiveButtonPanel.add(BorderLayout.CENTER, mPassiveSearch);
		passiveButtonPanel.add(BorderLayout.EAST, mPassiveButton);
		passiveButtonPanel.add(BorderLayout.SOUTH, mPassiveStatus);
		
		JPanel	passiveDownloadButtonsPanel = new JPanel(new FlowLayout());
		JButton	passiveDownloadButton = new JButton("Download");
		JButton	passiveClearButton = new JButton("Clear");
		passiveDownloadButton.addActionListener(new PassiveDownloadHandler());
		addRefreshComponent("ActionTransferDownload", passiveDownloadButton);
		passiveClearButton.addActionListener(new PassiveClearHandler());
		passiveDownloadButtonsPanel.add(passiveDownloadButton);
		passiveDownloadButtonsPanel.add(passiveClearButton);
		
		passiveSearchPanel.add(BorderLayout.NORTH, passiveButtonPanel);
		passiveSearchPanel.add(BorderLayout.CENTER, new JScrollPane(mPassiveSearchTable));
		passiveSearchPanel.add(BorderLayout.SOUTH, passiveDownloadButtonsPanel);
		passiveSearchPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(8, 8, 8, 8),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(8, 8, 8, 8)));
		
		monitorTablePanel.setPreferredSize(new Dimension(780, 200));
		passiveSearchPanel.setPreferredSize(new Dimension(780, 230));
		splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setDividerSize(6);
		splitPane.setTopComponent(monitorTablePanel);
		splitPane.setBottomComponent(passiveSearchPanel);

		mTabs.add(" Monitor Search ", splitPane);


		//
		//  Download Tab
		//

		JPanel	downloadTablePanel = new JPanel(new BorderLayout());
		mDownloadModel = new DownloadTableModel();
		mDownloadSorter = new TableSorter(mDownloadModel);
		mDownloadTable = new JTable(mDownloadSorter);
		mDownloadTable.getSelectionModel().addListSelectionListener(new DownloadSelectionHandler());
		mDownloadSorter.addMouseListenerToHeaderInTable(mDownloadTable);

		JPanel	downloadButtonsPanel = new JPanel(new FlowLayout());
		JButton	stopDownloadButton = new JButton("Stop");
		JButton	resumeButton = new JButton("Resume");
		JButton	removeDownloadButton = new JButton("Remove");
		JButton	showLogButton = new JButton("Show Log");
		addRefreshComponent("ActionDownloadStop", stopDownloadButton);
		addRefreshComponent("ActionDownloadResume", resumeButton);
		addRefreshComponent("ActionDownloadRemove", removeDownloadButton);
		addRefreshComponent("ActionDownloadShowLog", showLogButton);
		downloadButtonsPanel.add(stopDownloadButton);
		downloadButtonsPanel.add(resumeButton);
		downloadButtonsPanel.add(removeDownloadButton);
		downloadButtonsPanel.add(showLogButton);
		stopDownloadButton.addActionListener(new StopDownloadHandler());
		resumeButton.addActionListener(new ResumeDownloadHandler());
		removeDownloadButton.addActionListener(new RemoveDownloadHandler());
		showLogButton.addActionListener(new ShowDownloadLogHandler());

		downloadTablePanel.add(BorderLayout.NORTH, new JLabel("Downloads:"));
		downloadTablePanel.add(BorderLayout.CENTER, new JScrollPane(mDownloadTable));
		downloadTablePanel.add(BorderLayout.SOUTH, downloadButtonsPanel);
		downloadTablePanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(8, 8, 8, 8),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(8, 8, 8, 8)));

		mDownloadCandidateModel = new DownloadCandidateTableModel();
		mDownloadCandidate = new JTable(mDownloadCandidateModel);
		mDownloadCandidate.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		mDownloadCandidate.getSelectionModel().addListSelectionListener(new SelectionHandler());

		JPanel	downloadCandidateButtonsPanel = new JPanel(new FlowLayout());
		JButton	currCandidateButton = new JButton("Set As Current");
		JButton	removeCandidateButton = new JButton("Remove Candidate");
		JButton	searchCandidateButton = new JButton("Search...");
		JButton	stopSearchCandidateButton = new JButton("Stop");
		JButton	searchNewCandidateButton = new JButton("Search for new Candidates...");
		mAutoSearchCandidateCheck = new JCheckBox("Re-search at Startup", ServiceManager.sCfg.mAutoSearchCandidate);
		addRefreshComponent("ActionDownloadSetCandidateCurrent", currCandidateButton);
		addRefreshComponent("ActionDownloadRemoveCandidate", removeCandidateButton);
		addRefreshComponent("ActionDownloadSearchCandidate", searchCandidateButton);
		addRefreshComponent("ActionDownloadStopSearchCandidate", stopSearchCandidateButton);
		addRefreshComponent("ActionDownloadSearchNewCandidate", searchNewCandidateButton);
		downloadCandidateButtonsPanel.add(currCandidateButton);
		downloadCandidateButtonsPanel.add(removeCandidateButton);
		downloadCandidateButtonsPanel.add(searchCandidateButton);
		downloadCandidateButtonsPanel.add(stopSearchCandidateButton);
		downloadCandidateButtonsPanel.add(searchNewCandidateButton);
		downloadCandidateButtonsPanel.add(mAutoSearchCandidateCheck);


		currCandidateButton.addActionListener(new SetCandidateCurrentHandler());
		removeCandidateButton.addActionListener(new RemoveCandidateHandler());
		searchCandidateButton.addActionListener(new SearchCandidateHandler());
		stopSearchCandidateButton.addActionListener(new StopSearchCandidateHandler());
		searchNewCandidateButton.addActionListener( new SearchNewCandidateHandler());
		mAutoSearchCandidateCheck.addActionListener(new AutoSearchCandidateHandler());

		JPanel	downloadCandidatePanel = new JPanel(new BorderLayout());
		mCandidateLabelStatus = new JLabel("Download Candidates:  ");
		downloadCandidatePanel.add(BorderLayout.NORTH, mCandidateLabelStatus);
		downloadCandidatePanel.add(BorderLayout.CENTER, new JScrollPane(mDownloadCandidate));
		downloadCandidatePanel.add(BorderLayout.SOUTH, downloadCandidateButtonsPanel);
		downloadCandidatePanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(8, 8, 8, 8),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(8, 8, 8, 8)));

		downloadTablePanel.setPreferredSize(new Dimension(780, 300));
		downloadCandidatePanel.setPreferredSize(new Dimension(780, 200));

		splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setDividerSize(6);
		splitPane.setTopComponent(downloadTablePanel);
		splitPane.setBottomComponent(downloadCandidatePanel);

		mDownload = new JPanel(new BorderLayout());
		mDownload.add(BorderLayout.CENTER, splitPane);

		mTabs.add(" Download ", mDownload);

		//
		//  Upload Tab
		//

		JPanel	uploadTablePanel = new JPanel(new BorderLayout());
		mUploadModel = new UploadTableModel();
		mUploadSorter = new TableSorter(mUploadModel);
		mUploadTable = new JTable(mUploadSorter);
		mUploadTable.getSelectionModel().addListSelectionListener(new SelectionHandler());
		mUploadSorter.addMouseListenerToHeaderInTable(mUploadTable);
		uploadTablePanel.add(BorderLayout.NORTH, new JLabel("Uploads:"));
		uploadTablePanel.add(BorderLayout.CENTER, new JScrollPane(mUploadTable));

		JPanel	uploadButtonsPanel = new JPanel(new FlowLayout());
		JButton	abortUploadButton = new JButton("Abort Upload");
		JButton	clearUploadButton = new JButton("Clear Completed");
		addRefreshComponent("ActionShareAbort", abortUploadButton);
		addRefreshComponent("ActionShareClear", clearUploadButton);

		uploadButtonsPanel.add(abortUploadButton);
		uploadButtonsPanel.add(clearUploadButton);
		abortUploadButton.addActionListener(new AbortUploadHandler());
		clearUploadButton.addActionListener(new ClearUploadHandler());

		mUpload = new JPanel(new BorderLayout());
		mUpload.add(BorderLayout.CENTER, uploadTablePanel);
		mUpload.add(BorderLayout.SOUTH, uploadButtonsPanel);
		mUpload.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(8, 8, 8, 8),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(8, 8, 8, 8)));

		JPanel	shareTablePanel = new JPanel(new BorderLayout());
		mShareFilesModel = new ShareFilesModel();
		mShareFilesSorter = new TableSorter(mShareFilesModel);
		mShareFilesTable = new JTable(mShareFilesSorter);
		mShareFilesTable.getSelectionModel().addListSelectionListener(new SelectionHandler());
		mShareFilesSorter.addMouseListenerToHeaderInTable(mShareFilesTable);
		JPanel	shareButtonsPanel = new JPanel(new FlowLayout());
		JButton	shareRescanButton = new JButton("Rescan");
		shareButtonsPanel.add(shareRescanButton);
		shareRescanButton.addActionListener(new ShareRescanHandler());

		shareTablePanel.add(BorderLayout.NORTH, new JLabel("Files Sharing:"));
		shareTablePanel.add(BorderLayout.CENTER, new JScrollPane(mShareFilesTable));
		shareTablePanel.add(BorderLayout.SOUTH, shareButtonsPanel);
		shareTablePanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(8, 8, 8, 8),
					BorderFactory.createEtchedBorder()),
				BorderFactory.createEmptyBorder(8, 8, 8, 8)));

		mUpload.setPreferredSize(new Dimension(780, 230));
		shareTablePanel.setPreferredSize(new Dimension(780, 200));
		splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setDividerSize(6);
		splitPane.setTopComponent(mUpload);
		splitPane.setBottomComponent(shareTablePanel);

		mTabs.add(" Share ", splitPane);


		// Status Bar
		JPanel	statusBar = new JPanel(new BorderLayout());
		statusBar.setBorder(
				BorderFactory.createCompoundBorder(
					BorderFactory.createLoweredBevelBorder(),
					BorderFactory.createEmptyBorder(2, 2, 2, 2)));

		mSignalPanel = new SignalPanel();
		mStatusLabel = new JLabel();
		statusBar.add(BorderLayout.WEST, mSignalPanel);
		statusBar.add(BorderLayout.CENTER, mStatusLabel);


		getContentPane().add(BorderLayout.CENTER, mTabs);
		getContentPane().add(BorderLayout.SOUTH, statusBar);

		StatusChangedListener	statusListener = new StatusChangedListener();
		mHostMgr.addStatusChangedListener(statusListener);
		mShareMgr.addStatusChangedListener(statusListener);
		mDownloadMgr.addStatusChangedListener(statusListener);
		mHostMgr.addHostCountChangedListener(new HostCountChangedListener());
	}


	private void setupCellRender(JTable table, TableModel model)
	{
		int	colCount = model.getColumnCount();
		for (int i = 0; i < colCount; i++)
		{
			TableColumn	column = table.getColumn(model.getColumnName(i));
			column.setCellRenderer(mCellRenderer);
		}
	}


	private void setupPopupMenus()
	{
		mHostPopupMenu = new JPopupMenu();
		populatePopupMenu(mHostPopupMenu, "HostTable.PopupMenu");

		mCatcherPopupMenu = new JPopupMenu();
		populatePopupMenu(mCatcherPopupMenu, "CatcherTable.PopupMenu");
		
		mSearchResultPopupMenu = new JPopupMenu();
		populatePopupMenu(mSearchResultPopupMenu, "SearchResultTable.PopupMenu");

		mIrcPopupMenu = new JPopupMenu();
		populatePopupMenu(mIrcPopupMenu, "IrcChannelTable.PopupMenu");

		mIrcUserPopupMenu = new JPopupMenu();
		populatePopupMenu(mIrcUserPopupMenu, "IrcUserList.PopupMenu");
	}


	public void setTitle()
	{
		String	str = Res.getStr("Main.Title");

		if (mNetworkMgr.getJoined())
		{
			str += " - " + ServiceManager.sCfg.mCurrentNetwork;
		}

		if (ServiceManager.sCfg.mProxyUse)
		{
			str += "  (via Proxy Server)";
		}

		super.setTitle(str);
	}


	public void selectTab(int index)
	{
		mTabs.setSelectedIndex(index);
	}


	public boolean isNetTabSelected()
	{
		return (mTabs.getSelectedIndex() == 0);
	}


	public boolean isSearchTabSelected()
	{
		return (mTabs.getSelectedIndex() == 1);
	}


	public boolean isMonitorTabSelected()
	{
		return (mTabs.getSelectedIndex() == 2);
	}


	public boolean isDownloadTabSelected()
	{
		return (mTabs.getSelectedIndex() == 3);
	}


	public boolean isUploadTabSelected()
	{
		return (mTabs.getSelectedIndex() == 4);
	}


	public void removeHost()
	{
		int				rowCount = mNetTable.getSelectedRowCount();

		if (rowCount <= 0 || rowCount > mHostMgr.getHosts().size())
			return;

		int[]			rows = mNetTable.getSelectedRows();
		int[]			orderedRows = new int[rows.length];

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			if (row >= mHostMgr.getHosts().size())
				return;

			orderedRows[i] = mNetSorter.indexes[row];
		}

		// Bubble sort to assure ascending order.
		for (int i = orderedRows.length - 1; i > 0; i--)
		{
			for (int j = 0; j < i; j++)
			{
				if (orderedRows[j] > orderedRows[j+1])
				{
					int	tmp = orderedRows[j];
					orderedRows[j] = orderedRows[j+1];
					orderedRows[j+1] = tmp;
				}
			}
		}

		// Delete from the highest index to the lowest index.
		for (int i = orderedRows.length - 1; i >= 0; i--)
		{
			mHostMgr.removeHost(orderedRows[i]);
		}

		fireTableChanged(mNetTable, mNetModel);

		refreshAllActions();
	}


	public void updateStat()
	{
		mMsgManager.resetMyMsgInit();
		ServiceManager.getHostManager().sendMsgToHosts(mMsgManager.getMyMsgInit());
		ServiceManager.getHostManager().resetStat();
	}


	public void connectHost()
	{
		String		hostAddr = mConnectHostText.getText().trim();

		if (hostAddr.length() > 0)
		{
			StringTokenizer	tokens = new StringTokenizer(hostAddr, "\n");
			String			firstHost = tokens.nextToken();
			boolean			moreThanOne = false;

			// Add new host and connect.
			int				index = mHostMgr.addOutgoingHost(firstHost);
			if (index != -1)
			{
				mHostMgr.connectHost(index);
				fireTableChanged(mNetTable, mNetModel);
			}

			while (tokens.hasMoreTokens())
			{
				mHostMgr.addHostCaughtForced(tokens.nextToken());
				moreThanOne = true;
			}

			if (moreThanOne)
				mConnectHostText.setText("");
		}
		else
		{
			// Connect to list of existing hosts.
			int		rowCount = mNetTable.getSelectedRowCount();

			if (rowCount <= 0)
				return;

			int[]	rows = mNetTable.getSelectedRows();

			for (int i = 0; i < rowCount; i++)
			{
				int			row = rows[i];

				if (row >= mHostMgr.getHosts().size())
					continue;
				row = mNetSorter.indexes[row];
				mHostMgr.connectHost(row);
			}
			fireTableChanged(mNetTable, mNetModel);
		}
	}


	public boolean isConnectSelected()
	{
		String	hostAddr = mConnectHostText.getText().trim();

		if (hostAddr.length() > 0)
		{
			return true;
		}
		return isConnectTableSelected();
	}


	public boolean isConnectTableSelected()
	{
		int		rowCount = mNetTable.getSelectedRowCount();

		return (rowCount > 0 && mHostMgr.getHosts().size() > 0);
	}


	public void disconnectHost()
	{
		int				rowCount = mNetTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;

		int[]			rows = mNetTable.getSelectedRows();

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			if (row >= mHostMgr.getHosts().size())
				continue;
			row = mNetSorter.indexes[row];
			mHostMgr.disconnectHost(row);
		}
		fireTableChanged(mNetTable, mNetModel);
		refreshAllActions();
	}


	public boolean isDisconnectSelected()
	{
		int		rowCount = mNetTable.getSelectedRowCount();

		return (rowCount > 0 && mHostMgr.getHosts().size() > 0);
	}


	public void ignoreHost()
	{
		int				rowCount = mNetTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;

		int[]			rows = mNetTable.getSelectedRows();

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			if (row >= mHostMgr.getHosts().size())
				continue;
			row = mNetSorter.indexes[row];
			String		ip = ((Host)mHostMgr.getHosts().elementAt(row)).getHostAddr();
			ServiceManager.sCfg.mNetIgnoredHosts.addElement(Cfg.ip2parts(ip));
			mHostMgr.disconnectHost(row);
		}
		ServiceManager.sCfg.save();
		fireTableChanged(mNetTable, mNetModel);
		refreshAllActions();
	}


	void toggleAutoCleanup()
	{
		ServiceManager.sCfg.mAutoCleanup = mAutoCleanupCheck.isSelected();
		ServiceManager.sCfg.save();
	}


	void toggleAutoConnect()
	{
		ServiceManager.sCfg.mAutoConnect = mAutoConnectCheck.isSelected();
		ServiceManager.sCfg.save();
	}


	void updateStatusBar()
	{
		StringBuffer	buf = new StringBuffer();

		buf.append("  Connected Hosts: ");
		buf.append(mHostMgr.getConnectedHostCount());
		buf.append("    Uploads: ");
		buf.append(mShareMgr.getUploadingCount());
		buf.append(", ");
		buf.append(mHostMgr.getStatUploadCount());
		buf.append("    Downloads: ");
		buf.append(mDownloadMgr.getDownloadingCount());
		buf.append(", ");
		buf.append(mHostMgr.getStatDownloadCount());

		mStatusLabel.setText(buf.toString());
	}


	public boolean getSearchTextTyped()
	{
		return mSearchTextTyped;
	}


	public void search()
	{
		String		searchStr = mSearchText.getText().trim();
		String		maxSearchStr = mMaxSearch.getText().trim();
		String		minSpeedStr = mMinSpeed.getText().trim();
		int			minSpeed;
		String		searchFilter = mSearchFilterText.getText().trim();

		ServiceManager.sCfg.mSearchLastSearch = searchStr;
		ServiceManager.sCfg.mSearchMaxSearch = Integer.parseInt(maxSearchStr);
		ServiceManager.sCfg.mSearchMinSpeed = Integer.parseInt(minSpeedStr);
		ServiceManager.sCfg.mSearchFilter = searchFilter;
		ServiceManager.sCfg.save();

		if (searchStr.length() == 0)
			return;

		try
		{
			minSpeed = Integer.parseInt(minSpeedStr);
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(this, "Minimum speed should be an integer.", "Invalid Minimum Speed", JOptionPane.ERROR_MESSAGE);
			return;
		}

		mSearchTextTyped = false;
		refreshAllActions();

		mMsgManager.sendSearch(minSpeed, searchStr, this);
		mSearchResultCombo.setSelectedIndex(0);
		mSearchResultModel.fireListDataChanged();
	}


	public void stopSearch()
	{
		Search	search = getSelectedSearch();
		if (search != null)
		{
			search.stopSearching();
		}
	}


	public Search getSelectedSearch()
	{
		int		searchIndex = mSearchResultCombo.getSelectedIndex();
		if (searchIndex >= 0 && searchIndex < mMsgManager.getSearchCount())
		{
			return mMsgManager.getSearch(searchIndex);
		}

		return null;
	}


	private void updateSearchResultLabel()
	{
		Search	search = getSelectedSearch();
		if (search != null)
		{
			updateSearchResultLabel(search);
			return;
		}

		mSearchStopButton.setEnabled(false);
		refreshAllActions();
		mSearchResultLabel.setText("Search Result:");
	}


	private void updateSearchResultLabel(Search search)
	{
		String		str = "";
		Vector		results = search.getQueryResults();
		str += results.size() + " returned";

		if (search.getSearching())
			str += ", searching...";
		mSearchStopButton.setEnabled(search.getSearching());
		refreshAllActions();

		mSearchResultLabel.setText("Search Result:    " + str);
	}


	public void download()
	{
		Search	search = getSelectedSearch();
		if (search == null)
			return;
		Vector			results = search.getQueryResults();
		
		int				rowCount = mSearchTable.getSelectedRowCount();
		if (rowCount <= 0)
			return;

		int[]			rows = mSearchTable.getSelectedRows();
		int				addedCount = 0;

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			row = mSearchSorter.indexes[row];
			RemoteFile	rfile = (RemoteFile)results.elementAt(row);
			RemoteFile	dfile = new RemoteFile(rfile);
			if (!mDownloadMgr.addFileToDownload(dfile))
			{
				String	msg = "Another file with the same name '" + dfile.getFilename() + "' is being downloading now.\n";
				msg += "Please wait until the other file has completed its download.";
				JOptionPane.showMessageDialog(ServiceManager.getManager().getMainFrame(),
											  msg, "Duplicate File", JOptionPane.INFORMATION_MESSAGE);
			}
			else
			{
				addedCount++;
				rfile.setDownloaded(true);
			}
		}

		if (addedCount > 0)
		{
			mTabs.setSelectedIndex(3);
		}
		refreshAllActions();
	}


	void addAsCandidate()
	{
		Search	search = getSelectedSearch();
		if (search == null)
			return;
		Vector			results = search.getQueryResults();
		
		int				rowCount = mSearchTable.getSelectedRowCount();
		if (rowCount <= 0)
			return;

		int[]			rows = mSearchTable.getSelectedRows();
		int 			row = rows[0];
		row = mSearchSorter.indexes[row];
		RemoteFile	rfile = (RemoteFile)results.elementAt(row);
		RemoteFile	dfile = new RemoteFile(rfile);

		DlgAddAsCandidate	dlg = new DlgAddAsCandidate(this, dfile);
		dlg.setVisible(true);
		if (dlg.getCancel())
			return;

		DownloadFile	download = dlg.getDownloadFile();
		if (download == null)
			return;
		download.addRemoteCandidate(dfile);

		mTabs.setSelectedIndex(3);
		fireTableChanged(mDownloadCandidate, mDownloadCandidateModel);
		refreshAllActions();
	}


	public void copyURL()
	{
		Search		search = getSelectedSearch();
		if (search == null)
			return;
		Vector		results = search.getQueryResults();
		
		int			rowCount = mSearchTable.getSelectedRowCount();
		if (rowCount < 1)
		{
			return;
		}

		int[]		rows = mSearchTable.getSelectedRows();
		int			row = mSearchSorter.indexes[rows[0]];
		RemoteFile	rfile = (RemoteFile)results.elementAt(row);

		String		url = URLUtil.encodeURL(rfile.getURL());
		Toolkit.getDefaultToolkit().getSystemClipboard().setContents
				(new StringSelection(url), null);
	}


	public void launchBrowserOnURL()
	{
		RemoteFile	rfile;

		if (isSearchTabSelected())
		{
			Search		search = getSelectedSearch();
			if (search == null)
				return;

			if (mSearchTable.getSelectedRowCount() < 1)
				return;

			int[]		rows = mSearchTable.getSelectedRows();
			int			row = mSearchSorter.indexes[rows[0]];

			Vector		results = search.getQueryResults();
			rfile = (RemoteFile)results.elementAt(row);
		}
		else if (isMonitorTabSelected())
		{
			if (mPassiveSearchTable.getSelectedRowCount() < 1)
				return;

			int[]		rows = mPassiveSearchTable.getSelectedRows();
			int			row = mPassiveSearchSorter.indexes[rows[0]];
			Vector		results = mMsgManager.getPassiveResults();
			rfile = (RemoteFile)results.elementAt(row);
		}
		else
		{
			return;
		}

		String		url = URLUtil.encodeURL(rfile.getURL());
		try
		{
			ServiceManager.exec(ServiceManager.sCfg.mBrowserExe + " " + url);
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(this, "Failed to launch browser  " + e.getMessage(), "Can't Launch Browser", JOptionPane.ERROR_MESSAGE);
		}
	}


	public void launchBrowserOnHost()
	{
		RemoteFile	rfile;

		if (isSearchTabSelected())
		{
			Search		search = getSelectedSearch();
			if (search == null)
				return;

			if (mSearchTable.getSelectedRowCount() < 1)
				return;

			int[]		rows = mSearchTable.getSelectedRows();
			int			row = mSearchSorter.indexes[rows[0]];

			Vector		results = search.getQueryResults();
			rfile = (RemoteFile)results.elementAt(row);
		}
		else if (isMonitorTabSelected())
		{
			if (mPassiveSearchTable.getSelectedRowCount() < 1)
				return;

			int[]		rows = mPassiveSearchTable.getSelectedRows();
			int			row = mPassiveSearchSorter.indexes[rows[0]];
			Vector		results = mMsgManager.getPassiveResults();
			rfile = (RemoteFile)results.elementAt(row);
		}
		else
		{
			return;
		}

		String		url = StrUtil.parseURLHostPort(rfile.getURL());
		if (url == null)
			return;

		try
		{
			ServiceManager.exec(ServiceManager.sCfg.mBrowserExe + " " + url);
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(this, "Failed to launch browser  " + e.getMessage(), "Can't Launch Browser", JOptionPane.ERROR_MESSAGE);
		}
	}


	public void filterDownloadHost()
	{
		Search	search = getSelectedSearch();
		if (search == null)
			return;
		Vector			results = search.getQueryResults();

		int				rowCount = mSearchTable.getSelectedRowCount();
		if (rowCount <= 0)
			return;

		int[]			rows = mSearchTable.getSelectedRows();
		int				addedCount = 0;

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			row = mSearchSorter.indexes[row];
			RemoteFile	rfile = (RemoteFile)results.elementAt(row);
			ServiceManager.sCfg.mFilteredSearchHosts.addElement(Cfg.ip2parts(rfile.getRemoteHost()));
		}
		ServiceManager.sCfg.save();
	}


	public int getResultSelectedCount()
	{
		return (mSearchTable.getSelectedRowCount());
	}


	public void passiveSearch()
	{
		if (mMsgManager.getPassiveSearching() == null)
		{
			String	searchStr = mPassiveSearch.getText().trim();

			mMsgManager.setPassiveSearching(searchStr);

			mPassiveButton.setText("Stop Passive Search");
			mPassiveStatus.setText("Passive Search Result (Monitoring...)");
		}
		else
		{
			mMsgManager.setPassiveSearching(null);
			mPassiveButton.setText("Start Passive Search");
			mPassiveStatus.setText("Passive Search Result");
		}
		refreshAllActions();
	}


	public void passiveDownload()
	{
		int				rowCount = mPassiveSearchTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;

		int[]			rows = mPassiveSearchTable.getSelectedRows();
		Vector			results = mMsgManager.getPassiveResults();

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			row = mPassiveSearchSorter.indexes[row];
			RemoteFile	rfile = (RemoteFile)results.elementAt(row);
			RemoteFile	dfile = new RemoteFile(rfile);
			mDownloadMgr.addFileToDownload(dfile);
		}

		mTabs.setSelectedIndex(3);
		refreshAllActions();
	}


	public void passiveCopyURL()
	{
		int				rowCount = mPassiveSearchTable.getSelectedRowCount();

		if (rowCount != 1)
		{
			// Cannot copy multiple URLs to the clipboard
			return;
		}

		int[]			rows = mPassiveSearchTable.getSelectedRows();
		Vector			results = mMsgManager.getPassiveResults();
		int				row = mPassiveSearchSorter.indexes[rows[0]];
		RemoteFile		rfile = (RemoteFile)results.elementAt(row);

		String url = URLUtil.encodeURL (rfile.getURL());
		Toolkit.getDefaultToolkit().getSystemClipboard().setContents
				(new StringSelection(url), null);
	}


	public void filterPassiveDownloadHost()
	{
		int				rowCount = mPassiveSearchTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;

		int[]			rows = mPassiveSearchTable.getSelectedRows();
		Vector			results = mMsgManager.getPassiveResults();

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			row = mPassiveSearchSorter.indexes[row];
			RemoteFile	rfile = (RemoteFile)results.elementAt(row);
			ServiceManager.sCfg.mFilteredSearchHosts.addElement(Cfg.ip2parts(rfile.getRemoteHost()));
		}
		ServiceManager.sCfg.save();
	}


	public boolean isPassiveResultSelected()
	{
		return (mPassiveSearchTable.getSelectedRowCount() > 0);
	}


	void passiveClearResult()
	{
		mMsgManager.resetPassiveResults();
		refreshAllActions();
	}


	public void stopDownload()
	{
		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;

		int[]			rows = mDownloadTable.getSelectedRows();

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			if (row >= mDownloadMgr.getDownloadCount())
				return;

			row = mDownloadSorter.indexes[row];
			mDownloadMgr.stopDownload(row);
		}
		refreshAllActions();
	}


	public boolean isDownloadSelected()
	{
		return (isDownloadTabSelected() &&
				mDownloadTable.getSelectedRowCount() > 0 &&
				mDownloadMgr.getDownloadCount() > 0);
	}


	public void resumeDownload()
	{
		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;

		int[]			rows = mDownloadTable.getSelectedRows();

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			if (row >= mDownloadMgr.getDownloadCount())
				return;

			row = mDownloadSorter.indexes[row];
			mDownloadMgr.resumeDownload(row);
		}
		refreshAllActions();
	}


	public void removeDownload()
	{
		synchronized (mDownloadMgr)
		{
			int				rowCount = mDownloadTable.getSelectedRowCount();

			if (rowCount <= 0)
				return;

			int[]			rows = mDownloadTable.getSelectedRows();
			int[]			orderedRows = new int[rows.length];

			for (int i = 0; i < rowCount; i++)
			{
				int			row = rows[i];
				if (row >= mDownloadMgr.getDownloadCount())
					return;

				orderedRows[i] = mDownloadSorter.indexes[row];
			}

			// Bubble sort to assure ascending order.
			for (int i = orderedRows.length - 1; i > 0; i--)
			{
				for (int j = 0; j < i; j++)
				{
					if (orderedRows[j] > orderedRows[j+1])
					{
						int	tmp = orderedRows[j];
						orderedRows[j] = orderedRows[j+1];
						orderedRows[j+1] = tmp;
					}
				}
			}

			// Delete from the highest index to the lowest index.
			for (int i = orderedRows.length - 1; i >= 0; i--)
			{
				mDownloadMgr.removeDownload(orderedRows[i]);
			}

			refreshAllActions();
		}
	}

	void removeCompletedDownloads()
	{
		if (ServiceManager.sCfg.mDownloadAutoRemoveCompleted)
		{
			synchronized (mDownloadMgr)
			{
				mDownloadMgr.removeCompleted();
			}
			refreshAllActions();
		}
	}


	public void showDownloadLog()
	{
		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;

		int[]			rows = mDownloadTable.getSelectedRows();
		int				row = rows[0];
		if (row >= mDownloadMgr.getDownloadCount())
			return;
		row = mDownloadSorter.indexes[row];

		DownloadFile	download = (DownloadFile)mDownloadMgr.getDownloadList().elementAt(row);

		DlgLog			dlg = new DlgLog(this, "Download Log", "Log for " + download.getFilename(), download.getLog());

		dlg.setVisible(true);

//		JOptionPane.showMessageDialog(this, download.getLog(), "Download Log", JOptionPane.INFORMATION_MESSAGE);
	}


	public boolean isCandidateSelected()
	{
		if (!isDownloadTabSelected())
			return false;

		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return false;
		int[]			rows = mDownloadTable.getSelectedRows();
		int				i = rows[0];
		if (i >= mDownloadMgr.getDownloadCount())
			return false;
		i = mDownloadSorter.indexes[i];

		Vector			list = mDownloadMgr.getDownloadList();
		DownloadFile	download = (DownloadFile)list.elementAt(i);
		int				row = mDownloadCandidate.getSelectedRow();

		return (row != -1);
	}


	public boolean isSearchCandidate()
	{
		if (!isDownloadTabSelected())
			return false;

		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return false;
		int[]			rows = mDownloadTable.getSelectedRows();
		int				i = rows[0];
		if (i >= mDownloadMgr.getDownloadCount())
			return false;
		i = mDownloadSorter.indexes[i];

		Vector			list = mDownloadMgr.getDownloadList();
		DownloadFile	download = (DownloadFile)list.elementAt(i);

		return (!download.getIsSearching());
	}


	public boolean isStopSearchCandidate()
	{
		if (!isDownloadTabSelected())
			return false;

		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return false;
		int[]			rows = mDownloadTable.getSelectedRows();
		int				i = rows[0];
		if (i >= mDownloadMgr.getDownloadCount())
			return false;
		i = mDownloadSorter.indexes[i];

		Vector			list = mDownloadMgr.getDownloadList();
		DownloadFile	download = (DownloadFile)list.elementAt(i);

		return (download.getIsSearching());
	}


	public void setCandidateCurrent()
	{
		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;
		int[]			rows = mDownloadTable.getSelectedRows();
		int				i = rows[0];
		if (i >= mDownloadMgr.getDownloadCount())
			return;
		i = mDownloadSorter.indexes[i];

		Vector			list = mDownloadMgr.getDownloadList();
		DownloadFile	download = (DownloadFile)list.elementAt(i);
		if (download.getStatus() == DownloadFile.sConnecting ||
			download.getStatus() == DownloadFile.sDownloading ||
			download.getStatus() == DownloadFile.sRequestPushTransfer)
			return;

		int				row = mDownloadCandidate.getSelectedRow();

		download.setCandidateCurrent(row);
	}


	public void removeCandidate()
	{
		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;
		int[]			rows = mDownloadTable.getSelectedRows();
		int				i = rows[0];
		if (i >= mDownloadMgr.getDownloadCount())
			return;
		i = mDownloadSorter.indexes[i];

		Vector			list = mDownloadMgr.getDownloadList();
		DownloadFile	download = (DownloadFile)list.elementAt(i);
		int				row = mDownloadCandidate.getSelectedRow();

		download.removeRemoteCandidate(row);
	}


	public void searchCandidate()
	{
		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;
		int[]			rows = mDownloadTable.getSelectedRows();
		int				i = rows[0];
		if (i >= mDownloadMgr.getDownloadCount())
			return;
		i = mDownloadSorter.indexes[i];

		Vector			list = mDownloadMgr.getDownloadList();
		DownloadFile	download = (DownloadFile)list.elementAt(i);

		DlgInput1Str	dlg = new DlgInput1Str(this,
											   "Search Additional Candidates",
											   "Search the Gnutella network for additional download candidates.  Matching remote file has to have the same file size.",
											   "Search Text:",
											   download.getSearchTerm());
		dlg.setVisible(true);
		if (!dlg.getCanceled())
		{
			download.setSearchTerm(dlg.getValue1().trim());
			try
			{
				mDownloadMgr.saveDownloadInfo();
			}
			catch (IOException e)
			{
				ServiceManager.log(e);
			}
			download.searchCandidate();
		}
	}


	void stopSearchCandidate()
	{
		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;
		int[]			rows = mDownloadTable.getSelectedRows();
		int				i = rows[0];
		if (i >= mDownloadMgr.getDownloadCount())
			return;
		i = mDownloadSorter.indexes[i];

		Vector			list = mDownloadMgr.getDownloadList();
		DownloadFile	download = (DownloadFile)list.elementAt(i);

		download.stopSearchCandidate();
	}


	private void updateCandidateLabel()
	{
		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;
		int[]			rows = mDownloadTable.getSelectedRows();
		int				i = rows[0];
		if (i >= mDownloadMgr.getDownloadCount())
			return;
		i = mDownloadSorter.indexes[i];

		Vector			list = mDownloadMgr.getDownloadList();
		DownloadFile	download = (DownloadFile)list.elementAt(i);
		Search			search = download.getSearchObj();

		String			str = "";
		if (download.getIsSearching() && search != null)
		{
			Vector		results = search.getQueryResults();
			str += results.size() + " returned";

			str += ", searching...";
		}

		mCandidateLabelStatus.setText("Download Candidates:  " + str);
		refreshAllActions();
	}

	public void searchNewCandidate()
	{
		int				rowCount = mDownloadTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;
		int[]			rows = mDownloadTable.getSelectedRows();
		int				i = rows[0];
		if (i >= mDownloadMgr.getDownloadCount())
			return;
		i = mDownloadSorter.indexes[i];

		Vector			list = mDownloadMgr.getDownloadList();
		DownloadFile	download = (DownloadFile)list.elementAt(i);
		if (download.getStatus() == DownloadFile.sConnecting ||
			download.getStatus() == DownloadFile.sDownloading ||
			download.getStatus() == DownloadFile.sRequestPushTransfer)
			return;

		download.searchNewCandidate();
	}

	public void abortUpload()
	{
		int				row = mUploadTable.getSelectedRow();
		if (row < 0)
			return;

		if (row >= mShareMgr.getUploadList().size())
			return;

		row = mUploadSorter.indexes[row];

		mShareMgr.abortUpload(row);
		refreshAllActions();
	}


	public boolean isUploadSelected()
	{
		return (isUploadTabSelected() &&
				mUploadTable.getSelectedRow() >= 0 &&
				mShareMgr.getUploadList().size() > 0);
	}


	public void clearUpload()
	{
		mShareMgr.clearUpload();
		fireTableChanged(mUploadTable, mUploadModel);
		refreshAllActions();
	}


	public void connectFromHostCatcher()
	{
		int				rowCount = mCatcherTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;

		int[]			rows = mCatcherTable.getSelectedRows();
		Vector			results = mHostMgr.getHostsCaught();

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			String		hostaddr = (String)results.elementAt(row);
			int			index = mHostMgr.addOutgoingHost(hostaddr);
			if (index != -1)
			{
				mHostMgr.connectHost(index);
			}
		}

		fireTableChanged(mNetTable, mNetModel);
	}


	void autoConnectFromHostCatcher()
	{
		if (!ServiceManager.sCfg.mAutoConnect)
			return;

		int		index = mCatcherTable.getSelectedRow();
		Vector	hosts = mHostMgr.getHostsCaught();

		if (index < 0 || index >= hosts.size())
		{
			selectCatcherTable(0);
			return;
		}

		String	hostname = (String)hosts.elementAt(index);
		if (hostname != null)
		{
			int	i = mHostMgr.addOutgoingHost(hostname);
			if (i != -1)
				mHostMgr.connectHost(i);
		}

		selectCatcherTable(index + 1);
	}


	void selectCatcherTable(int index)
	{
		Vector	hosts = mHostMgr.getHostsCaught();

		mCatcherTable.clearSelection();
		if (index < hosts.size())
		{
			mCatcherTable.addRowSelectionInterval(index, index);
			mCatcherTable.scrollRectToVisible(mCatcherTable.getCellRect(index, 0, true));
		}
	}


	public void invalidHost()
	{
		int				rowCount = mCatcherTable.getSelectedRowCount();

		if (rowCount <= 0)
			return;

		int[]			rows = mCatcherTable.getSelectedRows();
		Vector			results = mHostMgr.getHostsCaught();

		for (int i = 0; i < rowCount; i++)
		{
			int			row = rows[i];
			String		hostaddr = (String)results.elementAt(row);
			ServiceManager.sCfg.mNetInvalidHosts.addElement(Cfg.ip2parts(hostaddr));
		}
		ServiceManager.sCfg.save();
		refreshAllActions();
		fireTableChanged(mCatcherTable, mCatcherModel);
	}


	public boolean isConnectFromHostCatcherSelected()
	{
		int		rowCount = mCatcherTable.getSelectedRowCount();

		return (rowCount > 0 && mHostMgr.getHostsCaught().size() > 0);
	}


	public void resetHostCatcher()
	{
		mHostMgr.resetHostsCaught();
		selectCatcherTable(0);
	}

	public void refreshUserName()
	{
		mMyNameLabel.setText("        My User Name:  " + ServiceManager.sCfg.mUserName);
	}


	// Go through freaking hoops to work around the table selection bug in JDK 1.3.
	private void fireTableChanged(JTable table, AbstractTableModel model)
	{
		// Save the table selection before firing the table changed event.
		int		rowCount = table.getSelectedRowCount();
		int[]	rows = table.getSelectedRows();

		// Fire the event
		model.fireTableDataChanged();

		// Restore the table selection.
		if (rowCount > 0)
		{
			for (int i = 0; i < rows.length; i++)
			{
				if (rows[i] < table.getRowCount())
					table.addRowSelectionInterval(rows[i], rows[i]);
			}
		}
	}



	/**
	*	Class to handle the WindowClosing event on the main frame.
	*
	*/
	private class WindowHandler extends WindowAdapter
	{
		/**
		*	Just delegate to the ActionFileExit acion.
		*
		*/
		public void windowClosing(WindowEvent e)
		{
			doAction(MainFrame.this, "ActionFileExit");
		}
	}


	private class NetTableModel extends AbstractTableModel
	{
		public int getColumnCount()
		{
			return 6;
		}


		public int getRowCount()
		{
			return ServiceManager.getHostManager().getHosts().size();
		}


		public String getColumnName(int col)
		{
			switch (col)
			{
				case 0:
					return "Remote Host";
				case 1:
					return "Type";
				case 2:
					return "Received (Dropped)";
				case 3:
					return "Sent (Queued)";
				case 4:
					return "Ping Latency (ms)";
				case 5:
					return "Status";
			}
			return "";
		}


		public Object getValueAt(int row, int col)
		{
			Vector			hosts = mHostMgr.getHosts();

			if (row >= hosts.size())
				return "";

			Host			host = (Host)hosts.elementAt(row);

			switch (col)
			{
				case 0:
					return host.getHostAddr();
				case 1:
					return host.getTypeName();
				case 2:
					return String.valueOf(host.getReceivedCount() + " (" + String.valueOf(host.getDropCount()) + ")");
				case 3:
					return String.valueOf(host.getSentCount()) + " (" + String.valueOf(host.getSendQueueLength()) + ")";
				case 4:
					if (host.getLatency() == 99999)
						return "Timeout";
					else
						return String.valueOf(host.getLatency());
				case 5:
					return host.getStatusName();
			}

			return "";
		}


		public void setValueAt(Object value, int row, int col)
		{
			// Do nothing
		}


		public boolean isCellEditable(int row, int col)
		{
			return false;
		}


		public Class getColumnClass(int col)
		{
			return String.class;
		}

	}


	private class CatcherTableModel extends AbstractTableModel
	{
		public int getColumnCount()
		{
			return 1;
		}


		public int getRowCount()
		{
			HostManager		hm = ServiceManager.getHostManager();
			Vector			results = hm.getHostsCaught();
			return results.size();
		}


		public String getColumnName(int col)
		{
			return "Hosts Detected";
		}


		public Object getValueAt(int row, int col)
		{
			HostManager		hm = ServiceManager.getHostManager();
			Vector			results = hm.getHostsCaught();

			if (row >= results.size())
				return "";
			
			return results.elementAt(row);
		}


		public void setValueAt(Object value, int row, int col)
		{
			// Do nothing
		}


		public boolean isCellEditable(int row, int col)
		{
			return false;
		}


		public Class getColumnClass(int col)
		{
			return String.class;
		}

	}


	private class SearchTableModel extends AbstractTableModel
	{
		public int getColumnCount()
		{
			return 5;
		}


		public int getRowCount()
		{
			Search		search = getSelectedSearch();
			if (search == null)
				return 0;
			Vector		results = search.getQueryResults();
			return results.size();
		}


		public String getColumnName(int col)
		{
			switch (col)
			{
				case 0:
					return "Sharing Host";
				case 1:
					return "File";
				case 2:
					return "Extension";
				case 3:
					return "Size";
				case 4:
					return "Host Speed";
			}
			return "";
		}


		public Object getValueAt(int row, int col)
		{
			Search		search = getSelectedSearch();
			if (search == null)
				return "";
			Vector		results = search.getQueryResults();

			if (row >= results.size())
				return "";

			RemoteFile	rfile = (RemoteFile)results.elementAt(row);

			switch (col)
			{
				case 0:
					return rfile.getRemoteHost();
				case 1:
					return rfile.getShortname();
				case 2:
					return rfile.getFileExt() + (rfile.getDownloaded() ? " *" : "");
				case 3:
					return rfile.getFileSizeInt();
				case 4:
					return rfile.getSpeedInt();
			}
			return "";
		}


		public void setValueAt(Object value, int row, int col)
		{
			// Do nothing
		}


		public boolean isCellEditable(int row, int col)
		{
			return false;
		}


		public Class getColumnClass(int col)
		{
			switch (col)
			{
				case 0:
					return String.class;
				case 1:
					return String.class;
				case 2:
					return String.class;
				case 3:
					return Integer.class;
				case 4:
					return Integer.class;
			}
			return String.class;
		}

	}


	private class PassiveSearchTableModel extends AbstractTableModel
	{
		public int getColumnCount()
		{
			return 5;
		}


		public int getRowCount()
		{
			Vector			results = mMsgManager.getPassiveResults();
			return results.size();
		}


		public String getColumnName(int col)
		{
			switch (col)
			{
				case 0:
					return "Sharing Host";
				case 1:
					return "File";
				case 2:
					return "Extension";
				case 3:
					return "Size";
				case 4:
					return "Host Speed";
			}
			return "";
		}


		public Object getValueAt(int row, int col)
		{
			Vector			results = mMsgManager.getPassiveResults();

			if (row >= results.size())
				return "";

			RemoteFile		rfile = (RemoteFile)results.elementAt(row);

			switch (col)
			{
				case 0:
					return rfile.getRemoteHost();
				case 1:
					return rfile.getFilename();
				case 2:
					return rfile.getFileExt();
				case 3:
					return rfile.getFileSizeInt();
				case 4:
					return rfile.getSpeedInt();
			}
			return "";
		}


		public void setValueAt(Object value, int row, int col)
		{
			// Do nothing
		}


		public boolean isCellEditable(int row, int col)
		{
			return false;
		}


		public Class getColumnClass(int col)
		{
			switch (col)
			{
				case 0:
					return String.class;
				case 1:
					return String.class;
				case 2:
					return String.class;
				case 3:
					return Integer.class;
				case 4:
					return Integer.class;
			}
			return String.class;
		}

	}


	private class MonitorTableModel extends AbstractTableModel
	{
		public int getColumnCount()
		{
			return 3;
		}


		public int getRowCount()
		{
			FQueue			queue = mMsgManager.getNetSearchHistory();
			return queue.getCount();
		}


		public String getColumnName(int col)
		{
			switch (col)
			{
				case 0:
					return "Routed From";
				case 1:
					return "Search Text";
				case 2:
					return "Minimum Speed";
			}
			return "";
		}


		public Object getValueAt(int row, int col)
		{
			FQueue			queue = mMsgManager.getNetSearchHistory();

			if (row >= queue.getCount())
				return "";

			MsgQuery		msg = (MsgQuery)queue.getAt(row);
			if (msg == null)
				return "";

			switch (col)
			{
				case 0:
					Host	fromHost = msg.getHeader().getFromHost();
					return (fromHost == null ? "<Unknown>" : fromHost.getHostAddr());
				case 1:
					return msg.getSearchString();
				case 2:
					return String.valueOf(msg.getMinSpeed());
			}
			return "";
		}


		public void setValueAt(Object value, int row, int col)
		{
			// Do nothing
		}


		public boolean isCellEditable(int row, int col)
		{
			return false;
		}


		public Class getColumnClass(int col)
		{
			return String.class;
		}

	}


	private class DownloadTableModel extends AbstractTableModel
	{
		public int getColumnCount()
		{
			return 5;
		}


		public int getRowCount()
		{
			return mDownloadMgr.getDownloadCount();
		}


		public String getColumnName(int col)
		{
			switch (col)
			{
				case 0:
					return "Sharing Host";
				case 1:
					return "File";
				case 2:
					return "Size";
				case 3:
					return "Rate";
				case 4:
					return "Status";
			}
			return "";
		}


		public Object getValueAt(int row, int col)
		{
			Vector			list = mDownloadMgr.getDownloadList();

			if (row >= list.size())
				return "";

			DownloadFile	download = (DownloadFile)list.elementAt(row);
			RemoteFile		rfile = download.getCurrentRemoteFile();

			switch (col)
			{
				case 0:
					if (download.getRemoteAppName() == null)
						return rfile.getRemoteHost();
					else
						return rfile.getRemoteHost() + "  (" + download.getRemoteAppName() + ")";
				case 1:
				{
					if (download.getRetryCount() > 0)
						return rfile.getShortname() + " (retry " + download.getRetryCount() + ")";
					else
						return rfile.getShortname();
				}
				case 2:
				{
					long	currsize = download.getTransferredSize();
					long	fsize = download.getFileSize();
					int		percentage = (int)(currsize * 100 / (fsize == 0 ? 1 : fsize));
					return "" + currsize + " / " + fsize + " (" + percentage + "%)";
				}
				case 3:
				{
					long	currsize = download.getTransferredSize();
					long	fsize = download.getFileSize();
					int		rate = download.getTransferRate();
					int		timeremaining = (int)((fsize - currsize) / (rate + 1));
					return "" + rate + "/Sec, " + StrUtil.formatTime(timeremaining) + " left";
				}
				case 4:
					return download.getStatusName();
			}

			return "";
		}


		public void setValueAt(Object value, int row, int col)
		{
			// Do nothing
		}


		public boolean isCellEditable(int row, int col)
		{
			return false;
		}


		public Class getColumnClass(int col)
		{
			return String.class;
		}

	}



	private class DownloadCandidateTableModel extends AbstractTableModel
	{
		public int getColumnCount()
		{
			return 5;
		}


		public int getRowCount()
		{
			int				rowCount = mDownloadTable.getSelectedRowCount();

			if (rowCount <= 0)
				return 0;
			int[]			rows = mDownloadTable.getSelectedRows();
			int				i = rows[0];
			if (i >= mDownloadMgr.getDownloadCount())
				return 0;
			i = mDownloadSorter.indexes[i];

			Vector			list = mDownloadMgr.getDownloadList();
			DownloadFile	download = (DownloadFile)list.elementAt(i);
			Vector			candidates = download.getRemoteCandidates();

			return candidates.size();
		}


		public String getColumnName(int col)
		{
			switch (col)
			{
				case 0:
					return "Sharing Host";
				case 1:
					return "File";
				case 2:
					return "Size";
				case 3:
					return "Host Speed";
				case 4:
					return "In Use";
			}
			return "";
		}


		public Object getValueAt(int row, int col)
		{
			int				rowCount = mDownloadTable.getSelectedRowCount();

			if (rowCount <= 0)
				return "";
			int[]			rows = mDownloadTable.getSelectedRows();
			int				i = rows[0];
			if (i >= mDownloadMgr.getDownloadCount())
				return "";
			i = mDownloadSorter.indexes[i];

			Vector			list = mDownloadMgr.getDownloadList();
			DownloadFile	download = (DownloadFile)list.elementAt(i);
			Vector			candidates = download.getRemoteCandidates();
			if (row >= candidates.size())
				return "";
			RemoteFile		rfile = (RemoteFile)candidates.elementAt(row);

			switch (col)
			{
				case 0:
					return rfile.getRemoteHost();
				case 1:
					return rfile.getShortname();
				case 2:
					return rfile.getFileSizeInt();
				case 3:
					return rfile.getSpeedInt();
				case 4:
					return (download.getCurrCandidate() == row ? "Current" : "");
			}

			return "";
		}


		public void setValueAt(Object value, int row, int col)
		{
			// Do nothing
		}


		public boolean isCellEditable(int row, int col)
		{
			return false;
		}


		public Class getColumnClass(int col)
		{
			return String.class;
		}

	}



	private class UploadTableModel extends AbstractTableModel
	{
		public int getColumnCount()
		{
			return 5;
		}


		public int getRowCount()
		{
			Vector			list = mShareMgr.getUploadList();
			return (list.size());
		}


		public String getColumnName(int col)
		{
			switch (col)
			{
				case 0:
					return "Downloading Host";
				case 1:
					return "File";
				case 2:
					return "Size";
				case 3:
					return "Rate";
				case 4:
					return "Status";
			}
			return "";
		}


		public Object getValueAt(int row, int col)
		{
			Vector			list = mShareMgr.getUploadList();

			if (row >= list.size())
			{
				return "";
			}

			UploadFile		ufile = (UploadFile)list.elementAt(row);

			switch (col)
			{
				case 0:
					if (ufile.getRemoteAppName() == null)
						return ufile.getRemoteHost();
					else
						return ufile.getRemoteHost() + "  (" + ufile.getRemoteAppName() + ")";
				case 1:
					return ufile.getFilename();
				case 2:
				{
					long	currsize = ufile.getTransferredSize();
					long	fsize = ufile.getTransferLength();
					int		percentage = (int)(currsize * 100 / (fsize > 0 ? fsize : 1));
					return "" + currsize + " / " + fsize + " (" + percentage + "%)";
				}
				case 3:
				{
					int		rate = ufile.getTransferRate();
					int		timeLeft = (int)((ufile.getTransferLength() - ufile.getTransferredSize()) / (rate + 1));
					return "" + rate + "/Sec, " + StrUtil.formatTime(timeLeft) + " left";
				}
				case 4:
					return ufile.getStatusName();
			}

			return "";
		}


		public void setValueAt(Object value, int row, int col)
		{
			// Do nothing
		}


		public boolean isCellEditable(int row, int col)
		{
			return false;
		}


		public Class getColumnClass(int col)
		{
			return String.class;
		}

	}


	private class ShareFilesModel extends AbstractTableModel
	{
		public int getColumnCount()
		{
			return 4;
		}


		public int getRowCount()
		{
			return mShareMgr.getFileCount();
		}


		public String getColumnName(int col)
		{
			switch (col)
			{
				case 0:
					return "File";
				case 1:
					return "Alias";
				case 2:
					return "Search Count";
				case 3:
					return "Upload Count";
			}
			return "";
		}


		public Object getValueAt(int row, int col)
		{
			if (row >= mShareMgr.getFileCount())
			{
				return "";
			}

			ShareFile		file = mShareMgr.getFile(row);

			switch (col)
			{
				case 0:
					return file.getFile().getAbsolutePath();
				case 1:
					return file.getAlias() == null ? "" : file.getAlias();
				case 2:
					return new Integer(file.getSearchCount());
				case 3:
					return new Integer(file.getUploadCount());
			}

			return "";
		}


		public void setValueAt(Object value, int row, int col)
		{
			// Do nothing
		}


		public boolean isCellEditable(int row, int col)
		{
			return false;
		}


		public Class getColumnClass(int col)
		{
			switch (col)
			{
				case 2:
					return Integer.class;
				case 3:
					return Integer.class;
			}

			return String.class;
		}

	}

	private class SearchComboModel implements ComboBoxModel
	{
		private Vector		mListDataListeners = new Vector();
		private Object		mSelectedItem = "";
		
		
		public int getSize()
		{
			return mMsgManager.getSearchCount();
		}


		public Object getElementAt(int index)
		{
			Search		search = mMsgManager.getSearch(index);
			
			if (search == null)
			{
				return "";
			}

			return search.getDisplayStr();
		}


		public void addListDataListener(ListDataListener l)
		{
			mListDataListeners.addElement(l);
		}


		public void removeListDataListener(ListDataListener l)
		{
			mListDataListeners.removeElement(l);
		}


		public void setSelectedItem(Object anItem)
		{
			mSelectedItem = anItem;
		}


		public Object getSelectedItem()
		{
			return mSelectedItem;
		}


		void fireListDataChanged()
		{
			int				last = getSize() - 1;
			if (last < 0)
				return;

			ListDataEvent	event = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, last);
			for (int i = 0; i < mListDataListeners.size(); i++)
			{
				ListDataListener	l = (ListDataListener)mListDataListeners.elementAt(i);
				l.contentsChanged(event);
			}
		}
	}


	private class SearchComboActionHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			updateSearchResultLabel();
			fireTableChanged(mSearchTable, mSearchModel);
		}
	}


	private class DownloadSelectionHandler implements ListSelectionListener
	{
		public void valueChanged(ListSelectionEvent e)
		{
			if (mInRefreshHandler)
				return;

			updateCandidateLabel();
			fireTableChanged(mDownloadCandidate, mDownloadCandidateModel);
			refreshAllActions();
		}
	}


	private class SelectionHandler implements ListSelectionListener
	{
		public void valueChanged(ListSelectionEvent e)
		{
			refreshAllActions();
		}
	}


	private class ChangeHandler implements ChangeListener
	{
		public void stateChanged(ChangeEvent e)
		{
			refreshAllActions();
		}
	}


	private class ConnectHostHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			connectHost();
		}
	}


	private class UpdateStatHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			updateStat();
		}
	}


	private class AutoCleanupHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			toggleAutoCleanup();
		}
	}


	private class AutoConnectHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			toggleAutoConnect();
		}
	}


	private class ConnectCatcherHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			connectFromHostCatcher();
		}
	}
	


	private class ResetCatcherHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			resetHostCatcher();
		}
	}


	private class HostCountChangedListener implements IDataChangedListener
	{
		public void dataChanged(Object source)
		{
			Vector	hosts = (Vector)source;

			if (hosts.size() > 0)
			{
				mSignalPanel.setState(SignalPanel.sStateGreen);
			}
			else
			{
				mSignalPanel.setState(SignalPanel.sStateGrey);
			}
		}
	}


	private class StatusChangedListener implements IDataChangedListener
	{
		public void dataChanged(Object source)
		{
			updateStatusBar();
		}
	}


	private class ApplyFilterHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			ServiceManager.sCfg.mApplyFilterdHosts = mApplyFilterCheck.isSelected();
			ServiceManager.sCfg.save();
		}
	}


	private class SearchHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			search();
		}
	}


	private class stopSearchHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			stopSearch();
		}
	}


	private class ConnectHostKeyHandler implements KeyListener
	{
		public void keyTyped(KeyEvent event)
		{
		}

		public void keyReleased(KeyEvent event)
		{
			if (event.getKeyCode() == KeyEvent.VK_ENTER)
			{
				if (mConnectHostText.getText().trim().length() > 0)
					connectHost();
			}
			refreshAction("ActionHostConnect");
		}

		public void keyPressed(KeyEvent event)
		{
		}
	}


	private class SearchKeyHandler implements KeyListener
	{
		public void keyTyped(KeyEvent event)
		{
		}

		public void keyReleased(KeyEvent event)
		{
			if (event.getSource() == mSearchText && event.getKeyCode() == KeyEvent.VK_ENTER)
			{
				if (mSearchTextTyped)
				{
					search();
				}
			}
			else
			{
				mSearchTextTyped = true;
				refreshAllActions();
			}
		}

		public void keyPressed(KeyEvent event)
		{
		}
	}


	private class DownloadHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			download();
		}
	}


	private class AddAsCandidateHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			addAsCandidate();
		}
	}


	private class PassiveSearchHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			passiveSearch();
		}
	}


	private class PassiveClearHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			passiveClearResult();
		}
	}


	private class PassiveDownloadHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			passiveDownload();
		}
	}


	private class StopDownloadHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			stopDownload();
		}
	}


	private class ResumeDownloadHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			resumeDownload();
		}
	}


	private class RemoveDownloadHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			removeDownload();
		}
	}


	private class ShowDownloadLogHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			showDownloadLog();
		}
	}


	private class SetCandidateCurrentHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			setCandidateCurrent();
		}
	}


	private class RemoveCandidateHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			removeCandidate();
		}
	}


	private class SearchCandidateHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			searchCandidate();
		}
	}


	private class StopSearchCandidateHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			stopSearchCandidate();
		}
	}


	private class SearchNewCandidateHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			searchNewCandidate();
		}
	}
	

	private class AutoSearchCandidateHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			ServiceManager.sCfg.mAutoSearchCandidate = mAutoSearchCandidateCheck.isSelected();
			ServiceManager.sCfg.save();
		}
	}


	private class AbortUploadHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			abortUpload();
		}
	}


	private class ClearUploadHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			clearUpload();
		}
	}
	

	private class ShareRescanHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			ServiceManager.getShareManager().rescan();
		}
	}
	

	private class RefreshHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			mInRefreshHandler = true;

			if (mDownloadChanged)
			{
				fireTableChanged(mDownloadTable, mDownloadModel);
				mDownloadChanged = false;
			}

			if (mUploadChanged)
			{
				fireTableChanged(mUploadTable, mUploadModel);
				fireTableChanged(mShareFilesTable, mShareFilesModel);
				mUploadChanged = false;
			}

			fireTableChanged(mNetTable, mNetModel);

			StringBuffer	buf = new StringBuffer();
			buf.append(mHostMgr.getStatHosts()).append(" hosts\n");
			buf.append(mHostMgr.getStatFiles()).append(" files\n");
			buf.append(mHostMgr.getStatSizeStr()).append("\n");
			buf.append(mHostMgr.getStatMsgCount()).append(" messsages\n");
			buf.append(mHostMgr.getRate()).append(" bytes/second\n");
			buf.append(mHostMgr.getStatDropCount()).append(" dropped messsages\n");
			buf.append(mHostMgr.getStatTakenCount()/(mHostMgr.getStatMsgCount()+1)).append(" average hops\n");
			buf.append(mHostMgr.getStatDownloadCount()).append(" downloads\n");
			buf.append(mHostMgr.getStatUploadCount()).append(" uploads\n");

			mStats.setText(buf.toString());

			mInRefreshHandler = false;
		}
	}


	private class CellRenderer extends DefaultTableCellRenderer
	{
		public Component getTableCellRendererComponent(
			JTable table, Object value, boolean isSelected,
			boolean hasFocus, int row, int column)
		{
			this.setForeground(Color.black);
//			this.setBackground(Color.black);

			if (table == mNetTable)
			{
				Vector		hosts = mHostMgr.getHosts();

				if (row < hosts.size())
				{
					int		srow = mNetSorter.indexes[row];
					Host	host = (Host)hosts.elementAt(srow);
					switch (host.getStatus())
					{
						case Host.sStatusNotConnected:
							break;

						case Host.sStatusError:
//							this.setForeground(Color.darkGray);
							this.setForeground(Color.gray);
							break;

						case Host.sStatusConnecting:
						case Host.sStatusAccepting:
							this.setForeground(Color.red);
							break;

						case Host.sStatusConnected:
						{
							this.setForeground(Color.blue);
							switch (column)
							{
								case 2:
									if (host.dropPacketsInRed())
										this.setForeground(Color.red);
									break;
								case 3:
									if (host.sendQueueInRed())
										this.setForeground(Color.red);
									break;
								case 4:
									if (host.latencyInRed())
										this.setForeground(Color.red);
									break;
							}
							break;
						}
					}
				}
			}
			
			if (table == mDownloadTable)
			{
				Vector			list = mDownloadMgr.getDownloadList();

				if (row < list.size())
				{
					row = mDownloadSorter.indexes[row];
					DownloadFile	download = (DownloadFile)list.elementAt(row);
					switch (download.getStatus())
					{
						case DownloadFile.sError:
							this.setForeground(Color.darkGray);
							break;
						case DownloadFile.sDownloading:
							this.setForeground(Color.red);
							break;
						case DownloadFile.sCompleted:
							this.setForeground(Color.blue);
							break;
					}
				}
			}
			
			if (table == mUploadTable)
			{
				Vector			list = mShareMgr.getUploadList();

				if (row < list.size())
				{
					row = mUploadSorter.indexes[row];
					switch (((UploadFile)list.elementAt(row)).getStatus())
					{
						case UploadFile.sError:
						case UploadFile.sAborted:
							this.setForeground(Color.darkGray);
							break;
						case UploadFile.sUploading:
							this.setForeground(Color.red);
							break;
						case UploadFile.sCompleted:
							this.setForeground(Color.blue);
							break;
					}
				}
			}

			return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
		}
	}



	private class MouseHandler extends MouseAdapter implements MouseListener
	{
		public void mouseClicked(MouseEvent e)
		{
			if (e.getClickCount() == 2)
			{
				if (e.getSource() == mSearchTable)
				{
					download();
				}
				else if (e.getSource() == mPassiveSearchTable)
				{
					passiveDownload();
				}
			}
		}


		public void mouseReleased(MouseEvent e)
		{ 
			if (e.isPopupTrigger())
			{
				popupMenu((Component)e.getSource(), e.getX(), e.getY());
			}
		}


		public void mousePressed(MouseEvent e)
		{ 
			if (e.isPopupTrigger())
			{
				popupMenu((Component)e.getSource(), e.getX(), e.getY());
			}
		}


		private void popupMenu(Component source, int x, int y)
		{
			if (source == mNetTable)
			{
				mHostPopupMenu.show(source, x, y);
			}
			else if (source == mCatcherTable)
			{
				mCatcherPopupMenu.show(source, x, y);
			}
			else if (source == mSearchTable || source == mPassiveSearchTable)
			{
				mSearchResultPopupMenu.show(source, x, y);
			}
		}
	}


	// IHostChanged
	public void hostChanged()
	{
		fireTableChanged(mNetTable, mNetModel);
	}


	public void updateMyIP()
	{
		mMyIP.setText(ServiceManager.getListener().getMyServerAddress());
	}


	// IHostCaughtChanged
	public void hostCaughtChanged()
	{
		fireTableChanged(mCatcherTable, mCatcherModel);
	}


	// ISearch
	public void started(Search searchObj)
	{
		updateSearchResultLabel();
	}

	
	public void stopped(Search searchObj)
	{
		updateSearchResultLabel();
	}


	public void gotResult(Search searchObj)
	{
		Search	search = getSelectedSearch();
		if (search == null)
			return;

		if (search == searchObj)
		{
			updateSearchResultLabel(search);
			fireTableChanged(mSearchTable, mSearchModel);
		}
	}


	// IMonitorSearch
	public void searchArrived()
	{
		fireTableChanged(mMonitorTable, mMonitorModel);
	}


	public void passiveResultArrived()
	{
		fireTableChanged(mPassiveSearchTable, mPassiveSearchModel);
	}


	// IDownloadChanged
	public void downloadChanged()
	{
		mDownloadChanged = true;
	}


	// IDownloadCandidate
	public void candidatesHaveChanged()
	{
		updateCandidateLabel();
		fireTableChanged(mDownloadCandidate, mDownloadCandidateModel);
	}

	
	// IShareChanged
	public void shareChanged()
	{
		mUploadChanged = true;
	}

	void invokeLater(String methodName)
	{
		SwingUtilities.invokeLater(new Invoker(this, methodName));
	}


	void invokeLater1(String methodName, Object arg1)
	{
		Object[]	params = new Object[1];
		
		params[0] = arg1;
		SwingUtilities.invokeLater(new Invoker(this, methodName, params));
	}


	void invokeLater2(String methodName, Object arg1, Object arg2)
	{
		Object[]	params = new Object[2];
		
		params[0] = arg1;
		params[2] = arg2;
		SwingUtilities.invokeLater(new Invoker(this, methodName, params));
	}



	// IFind
	public void findInResult(boolean bMatchCase, boolean bFindDown, String searchText)
	{
		int				startingIndex;
		int				rowCount = mSearchTable.getSelectedRowCount();

		if (rowCount <= 0)
		{
			startingIndex = -1;
		}
		else
		{
			int[]		rows = mSearchTable.getSelectedRows();
			startingIndex = rows[0];
		}

		if (bFindDown)
		{
			startingIndex++;
		}
		else
		{
			startingIndex--;
		}

		if (!bMatchCase)
			searchText = searchText.toLowerCase();

		int	i = startingIndex;
		rowCount = mSearchSorter.getRowCount();
		int	colCount = mSearchSorter.getColumnCount();
		while (true)
		{
			if (bFindDown)
			{
				if (i >= rowCount)
					break;
			}
			else
			{
				if (i < 0)
					break;
			}
			
			for (int j = 0; j < colCount; j++)
			{
				Object	obj = mSearchSorter.getValueAt(i, j);
				if (obj == null)
					continue;

				int		index;

				if (bMatchCase)
					index = obj.toString().indexOf(searchText);
				else
					index = obj.toString().toLowerCase().indexOf(searchText);

				if (index != -1)
				{
					mSearchTable.clearSelection();
					mSearchTable.addRowSelectionInterval(i, i);
					mSearchTable.scrollRectToVisible(mSearchTable.getCellRect(i, 0, true));
					return;
				}
			}
			
			if (bFindDown)
			{
				i++;
			}
			else
			{
				i--;
			}
		}

		JOptionPane.showMessageDialog(this, "No more result found.", "Find", JOptionPane.ERROR_MESSAGE);
	}

}



