
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex.msg;


import java.net.*;

import phex.*;
import phex.interfaces.*;
import phex.utils.*;


public class MsgPushRequest implements IMsg
{
	private MsgHeader	mHeader;
	private GUID		mClientID;
	private int			mFileIndex;
	private InetAddress	mRequestHost;
	private short		mRequestPort;


	private MsgPushRequest()
	{
	}


	public MsgPushRequest(MsgHeader header)
	{
		setHeader(header);
		mClientID = new GUID();
		mFileIndex = 0;
		mRequestHost = null;
		mRequestPort = 0;
	}


	public void setHeader(MsgHeader header)
	{
		mHeader = header;
		mHeader.setFunction(MsgHeader.sPushRequest);
	}


	public MsgHeader getHeader()
	{
		return mHeader;
	}


	public GUID getClientID()
	{
		return mClientID;
	}


	public void setClientID(GUID ClientID)
	{
		this.mClientID = ClientID;
	}

	
	public int getFileIndex()
	{
		return mFileIndex;
	}


	public void setFileIndex(int mFileIndex)
	{
		this.mFileIndex = mFileIndex;
	}

	
	public InetAddress getRequestHost()
	{
		return mRequestHost;
	}


	public void setRequestHost(InetAddress RequestHost)
	{
		this.mRequestHost = RequestHost;
	}

	
	public short getRequestPort()
	{
		return mRequestPort;
	}


	public void setRequestPort(short RequestPort)
	{
		this.mRequestPort = RequestPort;
	}




	public void computeHeaderLen()
	{
		mHeader.setDataLen(getSize() - mHeader.getSize());
	}


	public int getSize()
	{
		return	mHeader.getSize() +
				mClientID.getSize() +
				4 + 4 + 2;
	}


	public void copy(MsgPushRequest b)
	{
		mHeader.copy(b.getHeader());

		mClientID = b.mClientID;
		mFileIndex = b.mFileIndex;
		mRequestHost = b.mRequestHost;
		mRequestPort = b.mRequestPort;
	}


	public int serialize(byte[] outbuf, int offset)
			throws Exception
	{
		offset = mHeader.serialize(outbuf, offset);
		offset = mClientID.serialize(outbuf, offset);
		offset = IOUtil.serializeIntLE(mFileIndex, outbuf, offset);	// Convert to Intel little-endian
		System.arraycopy(mRequestHost.getAddress(), 0, outbuf, offset, 4);
		offset += 4;
		offset = IOUtil.serializeShortLE(mRequestPort, outbuf, offset);

		return offset;
	}


	public int deserialize(byte[] inbuf, int offset)
			throws Exception
	{
		// Already read the header.
		offset = mClientID.deserialize(inbuf, offset);
		mFileIndex = IOUtil.deserializeIntLE(inbuf, offset);
		offset += 4;
		StringBuffer	strBuf = new StringBuffer();
		offset = IOUtil.deserializeIP(inbuf, offset, strBuf);
		mRequestHost = InetAddress.getByName(strBuf.toString());
		mRequestPort = IOUtil.deserializeShortLE(inbuf, offset);
		offset += 2;

		return offset;
	}


	public String toString()
	{
		return	"[" +
				mHeader + " " +
				"ClientID=" + mClientID + ", " +
				"FileIndex=" + mFileIndex + ", " +
				"RequestHost=" + mRequestHost + ", " +
				"RequestPort=" + mRequestPort +
				"]";
	}


}

