
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex.msg;


import java.net.*;
import java.util.*;

import phex.*;
import phex.interfaces.*;
import phex.utils.*;


public class MsgQueryResponse implements IMsg
{
	private MsgHeader		mHeader;
	private int				mNumRecords;			// byte;
	private short			mRemoteListeningPort;
	private InetAddress		mRemoteHost;
	private int				mRemoteHostSpeed;
	private GUID			mRemoteClientID;
	private Vector			mRecords = new Vector();	// MsgResRecord



	private MsgQueryResponse()
	{
	}


	public MsgQueryResponse(MsgHeader header)
	{
		setHeader(header);
		mNumRecords = 0;
		mRemoteListeningPort = 0;
		mRemoteHost = null;
		mRemoteHostSpeed = 0;
		mRemoteClientID = null;
	}


	public void setHeader(MsgHeader header)
	{
		mHeader = header;
		mHeader.setFunction(MsgHeader.sQueryResponse);
	}


	public MsgHeader getHeader()
	{
		return mHeader;
	}


	public int getNumRecords()
	{
		return mNumRecords;
	}


	public void setNumRecords(int NumRecords)
	{
		this.mNumRecords = NumRecords;
	}

	
	public short getRemoteListeningPort()
	{
		return mRemoteListeningPort;
	}


	public void setRemoteListeningPort(short RemoteListeningPort)
	{
		this.mRemoteListeningPort = RemoteListeningPort;
	}

	
	public InetAddress getRemoteHost()
	{
		return mRemoteHost;
	}


	public void setRemoteHost(InetAddress RemoteHost)
	{
		this.mRemoteHost = RemoteHost;
	}


	public String getRemoteHostStr()
	{
		return mRemoteHost.getHostAddress() + ":" + mRemoteListeningPort;
	}

	
	public int getRemoteHostSpeed()
	{
		return mRemoteHostSpeed;
	}


	public void setRemoteHostSpeed(int RemoteHostSpeed)
	{
		this.mRemoteHostSpeed = RemoteHostSpeed;
	}

	
	public GUID getRemoteClientID()
	{
		return mRemoteClientID;
	}


	public void setRemoteClientID(GUID RemoteClientID)
	{
		this.mRemoteClientID = RemoteClientID;
	}


	public void addMsgRecord(MsgResRecord record)
	{
		mRecords.addElement(record);
		mNumRecords = mRecords.size();
	}


	public MsgResRecord getMsgRecord(int i)
	{
		return (MsgResRecord)mRecords.elementAt(i);
	}





	public void computeHeaderLen()
	{
		mHeader.setDataLen(getSize() - mHeader.getSize());
	}


	public int getSize()
	{
		int	size = mHeader.getSize() + 4 + 2 + 4 + 4 + mRemoteClientID.getSize();

		for (int i = 0; i < mNumRecords; i++)
		{
			MsgResRecord	rec = (MsgResRecord)mRecords.elementAt(i);
			size += rec.getSize();
		}

		return size;
	}


	public void copy(MsgQueryResponse b)
	{
		mHeader.copy(b.getHeader());
		mNumRecords = mNumRecords;
		mRemoteListeningPort = mRemoteListeningPort;
		mRemoteHost = mRemoteHost;
		mRemoteHostSpeed = mRemoteHostSpeed;
		mRemoteClientID = mRemoteClientID;
		for (int i = 0; i < b.getNumRecords(); i++)
		{
			MsgResRecord	rec = new MsgResRecord();
			rec.copy(b.getMsgRecord(i));
			addMsgRecord(rec);
		}
	}


	public int serialize(byte[] outbuf, int offset)
			throws Exception
	{
		offset = mHeader.serialize(outbuf, offset);
		outbuf[offset++] = (byte)mNumRecords;
		offset = IOUtil.serializeShortLE(mRemoteListeningPort, outbuf, offset);
		System.arraycopy(mRemoteHost.getAddress(), 0, outbuf, offset, 4);
		offset += 4;
		offset = IOUtil.serializeIntLE(mRemoteHostSpeed, outbuf, offset);

		for (int i = 0; i < mNumRecords; i++)
		{
			offset = getMsgRecord(i).serialize(outbuf, offset);
		}

		offset = mRemoteClientID.serialize(outbuf, offset);

		return offset;
	}


	public int deserialize(byte[] inbuf, int offset)
			throws Exception
	{
		// Already read the header.
		
		byte	n = inbuf[offset++];
		mNumRecords = (n < 0 ? 256 + n : n);
		mRemoteListeningPort = IOUtil.deserializeShortLE(inbuf, offset);
		offset += 2;
		StringBuffer	strBuf = new StringBuffer();
		offset = IOUtil.deserializeIP(inbuf, offset, strBuf);
		mRemoteHost = InetAddress.getByName(strBuf.toString());
		mRemoteHostSpeed = IOUtil.deserializeIntLE(inbuf, offset);
		offset += 4;

		for (int i = 0; i < mNumRecords; i++)
		{
			MsgResRecord	rec = new MsgResRecord();
			offset = rec.deserialize(inbuf, offset);
			mRecords.addElement(rec);
		}

		if (mRemoteClientID == null)
			mRemoteClientID = new GUID();
		offset = mRemoteClientID.deserialize(inbuf, offset);
		
		return offset;
	}


	public String toString()
	{
		String	str =
				"[" +
				mHeader + " " +
				"NumRecords=" + mNumRecords + ", " +
				"RemoteListeningPort=" + mRemoteListeningPort + ", " +
				"RemoteHost=" + mRemoteHost.getHostAddress() + ", " +
				"RemoteHostSpeed=" + mRemoteHostSpeed + ", " +
				"RemoteClientID=" + mRemoteClientID + ", " +
				"Records=";

		for (int i = 0; i < mNumRecords; i++)
		{
			str += getMsgRecord(i) + " ";
		}
		str += "]";

		return str;
	}


}

