
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex;


import java.util.*;
import java.io.*;

import phex.config.*;
import phex.host.*;
import phex.msg.*;


public class NetworkManager
{
	private static final String	sRequestSignature = "GNUTELLA CONNECT";
	private static final String	sRequestVersion = "/0.4";

	
	private ServiceManager		mManager;
	private boolean				mJoined;


	NetworkManager()
	{
		mManager = ServiceManager.getManager();
		mJoined = ServiceManager.sCfg.mAutoJoin;
	}


	public void init()
	{
		if (getJoined())
		{
			try
			{
				mManager.getHostManager().resetHostsCaught();
				mManager.getHostManager().loadHosts();
			}
			catch (Exception e)
			{

			}
		}
	}


	public synchronized void joinNetwork()
	{
		try
		{
			mManager.getHostManager().resetHostsCaught();
			mManager.getHostManager().loadHosts();
		}
		catch (Exception e)
		{
//			e.printStackTrace();
		}

		mManager.getMainFrame().updateStat();
		mJoined = true;
		mManager.getMainFrame().setTitle();
		mManager.getMainFrame().refreshAllActions();
		mManager.getMainFrame().selectCatcherTable(0);
	}


	public synchronized void leaveNetwork()
	{
		mJoined = false;
		ServiceManager.getManager().getMainFrame().setTitle();
		ServiceManager.getManager().getMainFrame().refreshAllActions();
		ServiceManager.getHostManager().removeAll();

		MsgManager	mm = ServiceManager.getMsgManager();
		mm.stopAllSearches();
	}


	public boolean getJoined()
	{
		return mJoined;
	}


	public String getGreeting()
	{
		if (ServiceManager.sCfg.mCurrentNetwork.equals(ServiceManager.sCfg.sGeneralNetwork))
		{
			return sRequestSignature;
		}
		else
		{
			return "GNUTELLA PCONNECT " + ServiceManager.sCfg.mCurrentNetwork;
		}
	}


	public String getFullGreeting()
	{
		return getGreeting() + sRequestVersion;
	}


}



