
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex.download;


import java.io.*;
import java.net.*;
import java.util.*;

import phex.*;

public class RemoteFile implements Serializable
{
	private GUID			mRemoteClientID;
	private int				mFileIndex;
	private String			mFilename;
	private String			mShortname;
	private String			mFileExt = null;
	private int				mFileSize;
	private Integer			mFileSizeInt;
	private String			mRemoteHost;
	private int				mSpeed;
	private Integer			mSpeedInt;
	private boolean			mDownloaded;


	private RemoteFile()
	{
	}


	public RemoteFile(GUID remoteClientID, int fileIndex, String fileName, int fileSize, String remoteHost, int speed)
	{
		mRemoteClientID = remoteClientID;
		mFileIndex = fileIndex;
		mShortname = mFilename = fileName;
		mFileSize = fileSize;
		mFileSizeInt = new Integer(fileSize);
		mRemoteHost = remoteHost;
		mSpeed = speed;
		mSpeedInt = new Integer(speed);
		mDownloaded = false;

		// Some servants just return the path along with the filename.
		// Path info is platform-dependent.
		int	i = mFilename.lastIndexOf("/");
		if (i != -1)
		{
			mShortname = mFilename.substring(i + 1);
		}
		else
		{
			i = mFilename.lastIndexOf("\\");
			if (i != -1)
			{
				mShortname = mFilename.substring(i + 1);
			}
		}
	}


	public RemoteFile(RemoteFile b)
	{
		mRemoteClientID = b.mRemoteClientID;
		mFileIndex = b.mFileIndex;
		mFilename = b.mFilename;
		mShortname = b.mShortname;
		mFileSize = b.mFileSize;
		mFileSizeInt = b.mFileSizeInt;
		mRemoteHost = b.mRemoteHost;
		mSpeed = b.mSpeed;
		mSpeedInt = b.mSpeedInt;
		mDownloaded = b.mDownloaded;
	}


	public void copy(RemoteFile b)
	{
		mRemoteClientID = b.mRemoteClientID;
		mFileIndex = b.mFileIndex;
		mFilename = b.mFilename;
		mShortname = b.mShortname;
		mFileSize = b.mFileSize;
		mFileSizeInt = b.mFileSizeInt;
		mRemoteHost = b.mRemoteHost;
		mSpeed = b.mSpeed;
		mSpeedInt = b.mSpeedInt;
		mDownloaded = b.mDownloaded;
	}


	public GUID getRemoteClientID()
	{
		return mRemoteClientID;
	}


	public int getFileIndex()
	{
		return mFileIndex;
	}


 	public String getFilename()
	{
		return mFilename;
	}


 	public String getShortname()
	{
		return mShortname;
	}


	public String getFileExt()
	{
		if (mFileExt != null)
			return mFileExt;

		int		index = mFilename.lastIndexOf(".");

		if (index > -1)
		{
			mFileExt = mFilename.substring(index + 1, mFilename.length());
		}
		else
		{
			mFileExt = "";
		}

		return mFileExt;
	}


	public int getFileSize()
	{
		return mFileSize;
	}

	
	public Integer getFileSizeInt()
	{
		return mFileSizeInt;
	}

	
	public String getRemoteHost()
	{
		return mRemoteHost;
	}


	public int getSpeed()
	{
		return mSpeed;
	}


	public Integer getSpeedInt()
	{
		return mSpeedInt;
	}


	public boolean getDownloaded()
	{
		return mDownloaded;
	}


	public void setDownloaded(boolean downloaded)
	{
		this.mDownloaded = downloaded;
	}


	public String getURL()
	{
		return "http://" + mRemoteHost + "/get/" + mFileIndex + "/" + mFilename;
	}


	public String getGetRequest()
	{
		// GET /get/1/foo.txt HTTP/1.1\r\n
		return "GET /get/" + mFileIndex + "/" + mFilename + " HTTP/1.0\r\n";

		// Some servants can't handle URL encoded string.
		// return "GET /get/" + mFileIndex + "/" + URLEncoder.encode(mFilename) + " HTTP/1.0\r\n";
	}


	public String getRangeHeader(int startPos)
	{
		if (startPos == 0)
			return "";

//		return "Range: bytes=" + startPos + "-" (mFileSize - 1) + "\r\n";
		return "Range: bytes=" + startPos + "-\r\n";
	}


	public boolean equals(RemoteFile b)
	{
		return (mRemoteHost.equals(b.mRemoteHost) && mFileIndex == b.mFileIndex);
	}


	private void writeObject(java.io.ObjectOutputStream oos)
			throws IOException
	{
		// Use latest version to serialize.
		serialize1(oos);
	}


	private void readObject(java.io.ObjectInputStream ois)
			throws IOException, ClassNotFoundException
	{
		int	version = ois.readInt();
		switch (version)
		{
			case 1:
				deserialize1(ois);
				break;

			default:
				throw new IOException("Fail to deserialize datafile.  Unknown verison.");
		}
	}


	private void serialize1(ObjectOutputStream oos)
			throws IOException
	{
		// Write version 1
		oos.writeInt(1);

		oos.writeObject(mRemoteClientID);
		oos.writeInt(mFileIndex);
		oos.writeUTF(mFilename);
		oos.writeUTF(mShortname);
		oos.writeInt(mFileSize);
		oos.writeObject(mFileSizeInt);
		oos.writeUTF(mRemoteHost);
		oos.writeInt(mSpeed);
		oos.writeObject(mSpeedInt);
	}


	private void deserialize1(ObjectInputStream ois)
			throws IOException, ClassNotFoundException
	{
		mRemoteClientID = (GUID)ois.readObject();
		mFileIndex = ois.readInt();
		mFilename = ois.readUTF();
		mShortname = ois.readUTF();
		mFileSize = ois.readInt();
		mFileSizeInt = (Integer)ois.readObject();
		mRemoteHost = ois.readUTF();
		mSpeed = ois.readInt();
		mSpeedInt = (Integer)ois.readObject();
	}
	


	public int getComparisonRank(RemoteFile b)
	{
		if (mFileSize != b.mFileSize)
		{
			return 0;
		}

		if (mRemoteHost.equals(b.mRemoteHost) && mFileIndex == b.mFileIndex)
		{
			return 100;
		}

		return computeSimilarity(mFilename, b.mFilename);
	}


	private static int[]	sCharStatistic1 = new int[256];
	private static int[]	sCharStatistic2 = new int[256];


	private static int computeSimilarity(String str1, String str2)
	{
		synchronized (sCharStatistic1)
		{
			for (int i = 0; i < 256; i++)
			{
				sCharStatistic1[i] = 0;
				sCharStatistic2[i] = 0;
			}

			for (int i = 0; i < str1.length(); i++)
			{
				int	index = (int)str1.charAt(i);
				if (index > 255)
					index %= 256;
				sCharStatistic1[index]++;
			}
			for (int i = 0; i < str2.length(); i++)
			{
				int	index = (int)str2.charAt(i);
				if (index > 255)
					index %= 256;
				sCharStatistic2[index]++;
			}

			int	sameCount = 0;
			for (int i = 0; i < 256; i++)
			{
				if (sCharStatistic1[i] == 0 || sCharStatistic2[i] == 0)
					continue;
				if (sCharStatistic1[i] > sCharStatistic2[i])
				{
					sameCount += sCharStatistic2[i];
				}
				else
				{
					sameCount += sCharStatistic1[i];
				}
			}

			return (int)(((float)sameCount*2) / (str1.length() + str2.length()) * 100);
		}
	}


}


