
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex;


import java.util.*;

import phex.config.*;
import phex.download.*;
import phex.host.*;
import phex.interfaces.*;
import phex.msg.*;


public class Search
{
	private HostManager		mHostMgr = ServiceManager.getHostManager();
	private MsgManager		mMsgMgr = ServiceManager.getMsgManager();
	private int				mMinSpeed;
	private String			mQueryStr;
	private long			mFileSizeToMatch;
	private int				mMaxResult;
	private MsgQuery		mQuery;
	private Vector			mQueryResults;
	private boolean			mSearching = false;
	private long			mSearchStartTime;
	private ISearch			mSearchListener;
	private String			mDisplayStr;


	private Search()
	{
		// disable
	}


	public Search(int minSpeed, String queryStr, long fileSizeToMatch, ISearch searchListener)
	{
		mMinSpeed = minSpeed;
		mQueryStr = queryStr;
		mFileSizeToMatch = fileSizeToMatch;
		mMaxResult = ServiceManager.sCfg.mSearchMaxSearch;
		mQueryResults = new Vector();
		mSearchListener = searchListener;

		mQuery = new MsgQuery();
		mQuery.getHeader().setTTL(ServiceManager.sCfg.mNetTTL);
		mQuery.setMinSpeed((short)minSpeed);
		mQuery.setSearchString(queryStr);

		mDisplayStr = "[Search=" + queryStr +
					  "]    [Max Result=" + mMaxResult +
					  "]    [Min Speed=" + minSpeed + "]";
	}


	public int getMinSpeed()
	{
		return mMinSpeed;
	}

	
	public String getQueryStr()
	{
		return mQueryStr;
	}


	public String getDisplayStr()
	{
		return mDisplayStr;
	}

	
	public MsgQuery getQuery()
	{
		return mQuery;
	}

	
	public Vector getQueryResults()
	{
		return mQueryResults;
	}

	
	public boolean getSearching()
	{
		return mSearching;
	}

	
	public void startSearching()
	{
		mMsgMgr.checkAndAddMsgSeen(mQuery);		// add to seen list.
		mHostMgr.sendMsgToHosts(mQuery);
		mSearching = true;
		mSearchStartTime = System.currentTimeMillis();
		mSearchListener.started(this);
	}


	public void stopSearching()
	{
		mSearching = false;
		mSearchListener.stopped(this);
	}


	public void processResponse(Host remoteHost, MsgQueryResponse msg)
	{
		if (!mSearching)
			return;

		// Is it a response for the query?
		if (!msg.getHeader().getMsgID().equals(mQuery.getHeader().getMsgID()))
		{
			return;
		}

		if (!mHostMgr.isHostFiltered(msg.getRemoteHost().getHostAddress()))
		{
//			remoteHost.log("Got response to my query.  " + msg);

			String			hostStr = msg.getRemoteHostStr();
			int				speed = msg.getRemoteHostSpeed();
			GUID			rcID = msg.getRemoteClientID();
			MsgResRecord	rec;
			RemoteFile		rfile;

			for (int i = 0; i < msg.getNumRecords(); i++)
			{
				rec = msg.getMsgRecord(i);

				if (isResultFiltered(rec.getFilename()))
					continue;

				if (mQueryResults.size() >= mMaxResult)
				{
					stopSearching();
					return;
				}

				if (mFileSizeToMatch > 0 && mFileSizeToMatch != rec.getFileSize())
					return;

				rfile = new RemoteFile(rcID, rec.getFileIndex(), rec.getFilename(), rec.getFileSize(), hostStr, speed);
				mQueryResults.addElement(rfile);
				mSearchListener.gotResult(this);
			}
		}
	}



	private boolean isResultFiltered(String filename)
	{
		Vector	list = ServiceManager.sCfg.mSearchFilterTokens;

		for (int i = 0; i < list.size(); i++)
		{
			if (filename.indexOf((String)list.elementAt(i)) != -1)
			{
				return true;
			}
		}
		return false;
	}


}

