
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex;


import java.io.*;
import java.util.*;
import java.net.*;




public class ShareFile
{
	private File			mFile = null;
	private Integer			mFileIndex;
	private String			mAlias = null;
	private int				mSearchCount = 0;
	private int				mUploadCount = 0;
	private InetAddress		mRemoteHost = null;
	private int				mRemotePort = 0;
	private int				mRemoteFileNumber = 0;
	private ShareManager	mShareMgr;


	private ShareFile()
	{
		// private
	}


	public ShareFile(File file)
	{
		mFile = file;
		mShareMgr = ServiceManager.getShareManager();
		if (mFile == null)
			mFileIndex = new Integer(0);
	}


	public File getFile()
	{
		return mFile;
	}


	public Integer getFileIndex()
	{
		return mFileIndex;
	}


	public void setFileIndex(Integer index)
	{
		mFileIndex = index;
	}


	public long getLength()
	{
		return mFile == null ? 0 : mFile.length();
	}


	public String getAlias()
	{
		return mAlias;
	}


	public void setAlias(String alias)
	{
		mAlias = alias;
	}


	public int getSearchCount()
	{
		return mSearchCount;
	}


	public void incSearchCount(int amount)
	{
		mSearchCount += amount;
		mShareMgr.shareChanged();
	}


	public int getUploadCount()
	{
		return mUploadCount;
	}


	public void incUploadCount(int amount)
	{
		mUploadCount += amount;
		mShareMgr.shareChanged();
	}


	public String getEffectiveName()
	{
		if (mAlias == null)
		{
			return mFile == null ? "PHEX " + Res.getStr("Program.Version") : mFile.getName();
		}
		else
		{
			return mAlias;
		}
	}


	public void setRemoteShareFile(String remoteHost, int remotePort, int remoteFileNumber)
	{
		try
		{
			mRemoteHost = InetAddress.getByName(remoteHost);
		}
		catch (Exception e)
		{
			mRemoteHost = null;
			return;
		}
		mRemotePort = remotePort;
		mRemoteFileNumber = remoteFileNumber;
	}


	public InetAddress getRemoteShareFileHost()
	{
		return mRemoteHost;
	}


	public int getRemoteShareFilePort()
	{
		return mRemotePort;
	}


	public int getRemoteShareFileNumber()
	{
		return mRemoteFileNumber;
	}


}


