
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex.utils;


import java.util.*;
import java.math.*;

import phex.*;

public class StrUtil
{
	public static String getAppNameVersion()
	{
		return "PHEX/" + Res.getStr("Program.Version");
	}

	
	public static String padStringLeft(String str, int width)
	{
		return getPadStr(str, width) + str;
	}


	public static String padStringRight(String str, int width)
	{
		return str + getPadStr(str, width);
	}


	public static String getPadStr(String str, int width)
	{
		int	len = str.length();

		if (len >= width)
			return " ";

		return sBlanks.substring(0, width - len);
	}


	public static String convertToLocalSystemFilename(String filename)
	{
		String	os = System.getProperty("os.name").toUpperCase();
		if (os.indexOf("MAC") != -1)
		{
			return makeShortName(filename, 27);
		}
		else
		{
			return filename;
		}
	}


	public static String makeShortName(String name, int maxLen)
	{
		byte	xor = (byte)0xFF;
		int		len = name.length();

		if (len <= maxLen)
			return name;

		if (len > 255)
			len = 255;

		for (int i = 0; i < len; i++)
		{
			xor ^= ((byte)name.charAt(i));
		}

		byte[]	buf = new byte[1];

		buf[0] = (byte)len;
		String	lenPart = HexDec.convertBytesToHexString(buf);
		buf[0] = xor;
		String	xorPart = HexDec.convertBytesToHexString(buf);

		int		extIndex = name.lastIndexOf(".");
		if (extIndex >= name.length() - 4)
		{
			String	ext = name.substring(extIndex);
			int		off = name.length() - extIndex;
			name = name.substring(0, maxLen - 4 - off);	// -4 for 2 hex bytes of xor and 2 hex bytes of len
			return	name + lenPart + xorPart + ext;
		}
		else
		{
			name = name.substring(0, maxLen - 4);	// -4 for 2 hex bytes of xor and 2 hex bytes of len
			return	name + lenPart + xorPart;
		}
	}


	public static String getFileExt(String filename)
	{
		int		index = filename.lastIndexOf(".");

		if (index > -1)
		{
			return filename.substring(index + 1, filename.length());
		}
		else
		{
			return "";
		}
	}


	// Replace all occurrences of c with newString in buffer.
	// This returns the same buffer, now modified.
	public static StringBuffer replaceAllChar(StringBuffer buffer, char c, String newString)
	{
		// Go backwards so that when the new string is substituted,
		// the indexing of won't be messed up.
		for (int i = buffer.length() - 1; i >= 0; --i)
		{
			if (buffer.charAt (i) == c)
			{
				buffer.setCharAt(i, newString.charAt(0));
				if (newString.length() > 1)
					buffer.insert(i+1, newString.substring(1));
			}
		}

		return (buffer);
	}


	public static String replaceAllChar(String orgStr, char c, String replacingPhrase)
	{
		StringBuffer	buf = new StringBuffer(orgStr);
		replaceAllChar(buf, c, replacingPhrase);
		return buf.toString();
	}


	public static String[] hashtableKeys2Strings(Hashtable htable)
	{
		String[]		result = new String[htable.size()];
		int				i = 0;
		Enumeration		keys = htable.keys();

		while (keys.hasMoreElements())
		{
			result[i++] = (String)keys.nextElement();
		}

		return result;
	}


	// Input host addresses in the forms of:
	//	- host:port
	//	- host:port1-port2
	//	- host:port1,port2
	//	- host:port1-port2,port3
	public static Vector parseHostAddr(String addrs)
			throws Exception
	{
		Vector			resultAddrs = new Vector();

		if (addrs.indexOf("-") == -1 && addrs.indexOf(",") == -1)
		{
			resultAddrs.addElement(addrs);
			return resultAddrs;
		}

		int				index = addrs.indexOf(":");
		if (index == -1)
		{
			throw new Exception("Invalid host format.");
		}

		String			host = addrs.substring(0, index);
		String			ports = addrs.substring(index + 1);

		StringTokenizer	portTokens = new StringTokenizer(ports, ",");
		while (portTokens.hasMoreTokens())
		{
			String		port = portTokens.nextToken();
			index = port.indexOf("-");
			if (index == -1)
			{
				resultAddrs.addElement(host + ":" + port);
				continue;
			}

			String		port1 = port.substring(0, index);
			String		port2 = port.substring(index + 1);

			for (int i = Integer.parseInt(port1), end = Integer.parseInt(port2);
				 i <= end; i++)
			{
				resultAddrs.addElement(host + ":" + i);
			}
		}

		return resultAddrs;
	}


	public static String parseHost(String hostAddr)
	{
		int		index = hostAddr.indexOf(":");
		if (index == -1)
		{
			return hostAddr;
		}
		return hostAddr.substring(0, index);
	}


	public static String parsePort(String hostAddr)
	{
		int		index = hostAddr.indexOf(":");
		if (index == -1)
		{
			return hostAddr;
		}
		return hostAddr.substring(index + 1);
	}


	public static String parseURLHostPort(String url)
	{
		try
		{
			// http://host:port/ref
			int	index = url.indexOf("//");

			if (index == -1)
				return null;
			index = url.indexOf("/", index + 2);
			return url.substring(index);
		}
		catch (Exception e)
		{
			return null;
		}
	}


	public static String formatTime(int seconds)
	{
		int				hours = seconds / 3600;
		int				minutes = (seconds - hours * 3600) / 60;
		int				secs = seconds - hours * 3600 - minutes * 60;
		StringBuffer	buf = new StringBuffer(16);

		if (hours > 9)
		{
			buf.append(hours);
		}
		else
		{
			buf.append("0");
			buf.append(hours);
		}
		buf.append(":");

		if (minutes > 9)
		{
			buf.append(minutes);
		}
		else
		{
			buf.append("0");
			buf.append(minutes);
		}
		buf.append(":");

		if (secs > 9)
		{
			buf.append(secs);
		}
		else
		{
			buf.append("0");
			buf.append(secs);
		}

		return buf.toString();
	}


	public static String formatSizeBytes(long size)
	{
		StringBuffer	buf = new StringBuffer(16);


		if (size < s1K)
		{
			buf.append(size);
		}
		else if (size < s1MB)
		{
			int		value = (int)(size) >> 10;
			buf.append(value).append("K");
		}
		else if (size < s1GB)
		{
			BigDecimal	bvalue = new BigDecimal(((double)size) / s1MB);
			bvalue = bvalue.setScale(2, BigDecimal.ROUND_DOWN);
			buf.append(bvalue.toString()).append("MB");
		}
		else if (size < s1TB)
		{
			BigDecimal	bvalue = new BigDecimal(((double)size) / s1GB);
			bvalue = bvalue.setScale(2, BigDecimal.ROUND_DOWN);
			buf.append(bvalue.toString()).append("GB");
		}
		else
		{
			BigDecimal	bvalue = new BigDecimal(((double)size) / s1TB);
			bvalue = bvalue.setScale(2, BigDecimal.ROUND_DOWN);
			buf.append(bvalue.toString()).append("TB");
		}

		return buf.toString();
	}


	public static long			s1K  = 1024L;
	public static long			s1MB = 1024L * 1024L;
	public static long			s1GB = 1024L * 1024L * 1024L;
	public static long			s1TB = 1024L * 1024L * 1024L * 1024L;

	private static String		sBlanks = "                                                                                                           ";



	public static void main(String[] args)
	{
		try
		{
			System.out.println(parseHostAddr(args[0]));
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}


/*	
	public static void main(String[] args)
	{
		System.out.println(formatSizeBytes(599));
		System.out.println(formatSizeBytes(1023));
		System.out.println(formatSizeBytes(1024));
		System.out.println(formatSizeBytes(2000));
		System.out.println(formatSizeBytes(50000));
		System.out.println(formatSizeBytes(500000));
		System.out.println(formatSizeBytes(5000000));
		System.out.println(formatSizeBytes((1L << 20) + 1));
		System.out.println(formatSizeBytes((1L << 20) + 24200));
		System.out.println(formatSizeBytes((1L << 30) + 38000000));
		System.out.println(formatSizeBytes((2L << 40)));

		System.out.println(formatTime(1));
		System.out.println(formatTime(10));
		System.out.println(formatTime(100));
		System.out.println(formatTime(59));
		System.out.println(formatTime(60));
		System.out.println(formatTime(61));
		System.out.println(formatTime(120));
		System.out.println(formatTime(3599));
		System.out.println(formatTime(3600));
		System.out.println(formatTime(3601));
		System.out.println(formatTime(100000));
		System.out.println(formatTime(1000000000));
	}
*/


/*
	// Unit test
	public static void main(String[] args)
	{
//		System.out.println(computeSimilarity(args[0], args[1]));
		System.out.println(StrUtil.makeShortName("1234567890", 20));
		System.out.println(StrUtil.makeShortName("123456789012345678901234567890", 20));
		System.out.println(StrUtil.makeShortName("123456789012345678901234567890.txt", 20));
		System.out.println(StrUtil.makeShortName("123456789012345678901234567890.tx", 20));
		System.out.println(StrUtil.makeShortName("123456789012345678901234567890.txt1", 20));
		System.out.println(StrUtil.makeShortName("123456789012345678901234567890", 30));
		System.out.println(StrUtil.makeShortName("123456789012345678901234567890", 25));
		System.out.println(StrUtil.makeShortName("123456789012345678901234567891", 25));
	}
*/

}


