
/*
 *  PHEX - The pure-java Gnutella-servant.
 *  Copyright (C) 2000 William W. Wong
 *  williamw@jps.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package phex;


import java.io.*;
import java.net.*;
import java.util.*;

import phex.host.*;


public class UploadFile
{
	public static final int	sIdle = 0;
	public static final int	sQueued = 1;
	public static final int	sUploading = 3;
	public static final int	sCompleted = 4;
	public static final int	sError = 5;
	public static final int	sAborted = 6;


	private File			mFile;
	private String			mFilename;
	private Host			mRemoteHost;
	private long			mTransferLength;
	private long			mFileSize;
	private String			mRemoteAppName;
	private int				mStatus = sIdle;
	private String			mStatusText = "";
	private String			mLog = "";
	private long			mTransferredSize = 0;
	private long			mStartingTime = 0;
	private int				mTransferRate = 0;
	private ShareManager	mShareMgr;


	private UploadFile()
	{
		// disable
	}


	public UploadFile(File file, Host remoteHost, long transferLength, String remoteAppName)
	{
		mFile = file;
		mRemoteHost = remoteHost;
		mTransferLength = transferLength;
		mRemoteAppName = remoteAppName;
		
		mFilename = mFile.getName();
		mFileSize = mFile.length();
		mShareMgr = ServiceManager.getShareManager();
	}


	public File getFile()
	{
		return mFile;
	}


	public String getFilename()
	{
		return mFilename;
	}


	public Host getRemoteHost()
	{
		return mRemoteHost;
	}


	public long getTransferLength()
	{
		return mTransferLength;
	}


	public String getRemoteAppName()
	{
		return mRemoteAppName;
	}


	public int getStatus()
	{
		return mStatus;
	}


	public String getStatusName()
	{
		switch (mStatus)
		{
			case sIdle:
				return "Idle";

			case sQueued:
				return "Queued";

			case sUploading:
				return "Uploading";

			case sCompleted:
				return "Completed";

			case sError:
				return "Error " + mStatusText;

			case sAborted:
				return "Aborted";
		}
		return "Unknown";
	}


	public void setStatus(int status)
	{
		mStatus = status;
		mStatusText = "";
		mShareMgr.shareChanged();
	}


	public void setStatus(int status, String text)
	{
		mStatus = status;
		mStatusText = text;
		mShareMgr.shareChanged();
	}


	public void setLog(String log)
	{
		mLog = log;
	}


	public void appendLog(String log)
	{
		if (mLog != null)
			mLog += log + "\n";
		else
			mLog = log + "\n";
	}


	public String getLog()
	{
		return mLog;
	}


	public long getStartingTime()
	{
		return mStartingTime;
	}


	public void setStartingTime(long startingTime)
	{
		mStartingTime = startingTime;
	}


	public long getTransferredSize()
	{
		return mTransferredSize;
	}


	public void setTransferredSize(long TransferredSize)
	{
		mTransferredSize = TransferredSize;

		if (mStartingTime != 0)
		{
			long	secs = (System.currentTimeMillis() - mStartingTime) / 1000 + 1;
			mTransferRate = (int)(mTransferredSize / secs);
		}
		mShareMgr.shareChanged();
	}


	public int getTransferRate()
	{
		return mTransferRate;
	}


}


